/******************************************************************************

SOFTING AG
Richard-Reitzner-Allee 6
D-85540 Haar
Phone: ++49-89-4 56 56-0
Fax: ++49-89-4 56 56-3 99
http://www.softing.com

Copyright (C) SOFTING AG 2000-2005, All Rights Reserved

Version: 3.11.0.09.release

******************************************************************************/

#ifndef __BOOTV311_H__
#define __BOOTV311_H__

/******************************************************************************

! ! !  A T T E N T I O N  ! ! !

       This header file needs  B Y T E  structure alignment.

******************************************************************************/

/******************************************************************************
HOST COMMUNICATION
******************************************************************************/

typedef unsigned short   T_BOOTIF_SIGNAL;

#define BOOTIF_SIGNAL_ACKNOWLEDGE                       ((T_BOOTIF_SIGNAL) 0x0000)

/*--- HOST -> CONTROLLER ----------------------------------------------------*/

#define BOOTIF_SIGNAL_SERVICE_GET_DEVICE_INFO           ((T_BOOTIF_SIGNAL) 0x0001)
#define BOOTIF_SIGNAL_SERVICE_SET_LICENCE_DATA          ((T_BOOTIF_SIGNAL) 0x0002)
#define BOOTIF_SIGNAL_SERVICE_GET_LICENCE_DATA          ((T_BOOTIF_SIGNAL) 0x0003)
#define BOOTIF_SIGNAL_SERVICE_PROGRAM_FLASH             ((T_BOOTIF_SIGNAL) 0x0004)
#define BOOTIF_SIGNAL_SERVICE_TEST_INTERRUPT            ((T_BOOTIF_SIGNAL) 0x0005)
#define BOOTIF_SIGNAL_SERVICE_START_BUILT_IN_SELFTEST   ((T_BOOTIF_SIGNAL) 0x0006)
#define BOOTIF_SIGNAL_SERVICE_SET_FEATURES              ((T_BOOTIF_SIGNAL) 0x0007)
#define BOOTIF_SIGNAL_SERVICE_GET_FEATURES              ((T_BOOTIF_SIGNAL) 0x0008)
#define BOOTIF_SIGNAL_SERVICE_CALCULATE_FLASH_CHECKSUM  ((T_BOOTIF_SIGNAL) 0x0009)
#define BOOTIF_SIGNAL_SERVICE_START_BIST_WITH_ASPC2     ((T_BOOTIF_SIGNAL) 0x000A)

#define BOOTIF_SIGNAL_INITIALIZE_EEPROM                 ((T_BOOTIF_SIGNAL) 0x4B44) /* "DK" */
#define BOOTIF_SIGNAL_READ_EEPROM                       ((T_BOOTIF_SIGNAL) 0x4757) /* "WG" */

#define BOOTIF_SIGNAL_RESET_FIRMWARE                    ((T_BOOTIF_SIGNAL) 0x4000)

/*---------------------------------------------------------------------------*/

#define BOOT_IF_VERSION_NO_ID             0x0000
#define BOOT_IF_VERSION_IF_ID             0x0101
#define BOOT_IF_VERSION_BOOT_IF_ID        BOOT_IF_VERSION_IF_ID
#define BOOT_IF_VERSION_SERVICE_IF_ID     BOOT_IF_VERSION_IF_ID

/******************************************************************************
HOST INTERFACE
******************************************************************************/

/*--- boot select masks -----------------------------------------------------*/

#define START_UPDATE_FLASH_FIRMWARE  0x474F5250uL             /* "PROG" command to start PROGR firmware */
#define START_PROFIBUS_V5X_FIRMWARE  0x874B2D1EuL             /* command to start PROFIBUS V5x firmware */
#define START_PROFISTACK_FIRMWARE    0x47325044uL             /* command to start PROFIstack firmware */

#define START_BOOTBLOCK_FIRMWARE     0x544F4F42uL             /* "BOOT" command to start Bootblock firmware */

/*--- boot loader mask ------------------------------------------------------*/

#define BOOT_BLOCK_EXIST             0x544F4F42uL             /* "BOOT" Bootblock-Exist-Flag */

/*---------------------------------------------------------------------------*/


#define BOOTBLOCK_VERSION_SIZE                72
#define PROFIBUS_FIRMWARE_VERSION_SIZE        128



#define SBNSTRING 0xffff
#define SBNDATA   0x0000
#define SBNEOF    0x0001
#define SBNSTART  0x0003





#define PACK_BYTE_ALIGNMENT(Struct)   __attribute__ ((packed, aligned (2))) Struct


typedef struct _T_V311_BOOT_IF_DESCR   /* 232 bytes */
{
  volatile USIGN32      HostBootSelectMask;             /* must be located at offset 0x00 */

  unsigned char               Reserved1 [22];                 /* !!! DO NOT USE THIS AREA !!! */

  USIGN32                     LiveSign;

  volatile T_BOOTIF_SIGNAL    SignalFromHost;
  volatile T_BOOTIF_SIGNAL    SignalFromController;

  unsigned short              IfId;
  unsigned short              BootIfId;
  unsigned short              ServiceIfId;

  unsigned char               Reserved2 [64];

  USIGN32                     SerialNumber;
  unsigned short              DeviceId;
  unsigned short              HwRevision;

  volatile USIGN32            BootLoaderMask;                 /* must be located at offset 0x70 */
  USIGN32                     BootLoaderVersion;              /* must be located at offset 0x74 */

  char                        BootBlockVersionString [BOOTBLOCK_VERSION_SIZE];
  char                        FirmwareVersionString [PROFIBUS_FIRMWARE_VERSION_SIZE]; /* must be located at offset 0xC0 - 0x140*/

} PACK_BYTE_ALIGNMENT (T_V311_BOOT_IF_DESCR);


/*=== SERVICE INTERFACE =====================================================*/

/* softing profibus device idendifiers */
#define DEVICE_ID_PROFICARD1          0x0010
#define DEVICE_ID_PROFICARD2          0x0011
#define DEVICE_ID_PROFICARDM          0x0018
#define DEVICE_ID_PROFICARD2M         0x0019
#define DEVICE_ID_PROFIBOARD          0x0020
#define DEVICE_ID_PROFI104            0x0021
#define DEVICE_ID_PROFI104S           0x0022
#define DEVICE_ID_PROFIPCI            0x0028
#define DEVICE_ID_PROFIPCIS           0x0029

#define DEVICE_ID_PROFI_IP_MODULE     0x0089

/* softing profibus revision numbers */
#define DEVICE_REV_PROFICARD1_REV1    0x0100
#define DEVICE_REV_PROFICARD1_REV2    0x0101
#define DEVICE_REV_PROFIBOARD_REV1    0x0100
#define DEVICE_REV_PROFIBOARD_REV2    0x0101
#define DEVICE_REV_PROFIBOARD_REV3    0x0102
#define DEVICE_REV_PROFIBOARD_REV4    0x0103

/* includes 104D_KOP and 104D_STD same like PROFI104, but different chip-and addess select modes */
#define STRING_PROFI104_STD           0x44343031uL  /* 104D */

/* includes PBCOM same like PROFI104, but different chip-and addess select modes */
#define STRING_PROFI104_PBCOM         0x4F434250uL  /* PBCO */

typedef struct _T_V311_DEVICE_INFO
{
  USIGN32                     SerialNumber;
  unsigned short              DeviceId;
  unsigned short              HwRevision;

} PACK_BYTE_ALIGNMENT (T_V311_DEVICE_INFO);

/*---------------------------------------------------------------------------*/

#define FEATURE_IS_DISABLED           0x00u
#define FEATURE_IS_ENABLED            0x01u
#define FEATURE_DISABLE               0x02u
#define FEATURE_ENABLE                0x03u
#define FEATURE_DONT_CARE             0x04u

typedef struct _T_V311_LICENCE_DATA
{
  unsigned char               LicenceKey [32];

} PACK_BYTE_ALIGNMENT (T_V311_LICENCE_DATA);

/*---------------------------------------------------------------------------*/

#define PROFIBRD_IRQ_TO_HOST_NO   0x00u
#define PROFIBRD_IRQ_TO_HOST_5    0x10u
#define PROFIBRD_IRQ_TO_HOST_10   0x20u
#define PROFIBRD_IRQ_TO_HOST_11   0x30u
#define PROFIBRD_IRQ_TO_HOST_12   0x40u
#define PROFIBRD_IRQ_TO_HOST_15   0x50u

typedef struct _T_V311_TEST_INTERRUPT
{
  unsigned char               InterruptNumber;

} PACK_BYTE_ALIGNMENT (T_V311_TEST_INTERRUPT);

/*---------------------------------------------------------------------------*/

#define MAX_BINARY_LENGTH         2048u

#define TYPE_PROGRAM_DATA         0
#define TYPE_PROGRAM_END          1
#define TYPE_INITIALIZE_EEPROM    2

typedef struct _T_V311_PROGRAM_FLASH
{
  unsigned short              Type;

  USIGN32                     Offset;
  USIGN32                     Length;

  unsigned char               BinaryData [MAX_BINARY_LENGTH];

} PACK_BYTE_ALIGNMENT (T_V311_PROGRAM_FLASH);

/*---------------------------------------------------------------------------*/

typedef struct _T_V311_FLASH_CHECKSUM
{
  unsigned short    BootblockFlashChecksum;
  unsigned short    ProtocolFwFlashChecksum;

} PACK_BYTE_ALIGNMENT (T_V311_FLASH_CHECKSUM);

/*---------------------------------------------------------------------------*/

#define TESTCASE_VERIFY_FLASH_CHECKSUM             1u
#define TESTCASE_RAM_TEST                          2u
#define TESTCASE_DPRAM_TEST                        3u
#define TESTCASE_ASPC2_REGISTER_TEST               4u
#define TESTCASE_ASPC2_TEST                        5u
#define TESTCASE_ASPC2_BUS_TEST                    6u

typedef struct _T_V311_BUILTIN_SELF_TEST
{
  unsigned short    LiveSign;

  unsigned short    TestCase;
  unsigned short    Progess;

  unsigned short    RamValue;
  USIGN32           RamAddress;

} PACK_BYTE_ALIGNMENT (T_V311_BUILTIN_SELF_TEST);

/*---------------------------------------------------------------------------*/

#define BOOTIF_OK                                      0
#define BOOTIF_ERROR_INVALID_SERIAL_NUMBER            -1
#define BOOTIF_ERROR_EEPROM_WRITE_FAILED              -2
#define BOOTIF_ERROR_EEPROM_READ_FAILED               -3
#define BOOTIF_ERROR_INVALID_LENGTH                   -4
#define BOOTIF_ERROR_INVALID_OFFSET                   -5
#define BOOTIF_ERROR_FLASH_ERASE_ERROR                -6
#define BOOTIF_ERROR_FLASH_WRITE_ERROR                -7
#define BOOTIF_ERROR_FLASH_READ_ERROR                 -8
#define BOOTIF_ERROR_CHECK_SUM_ERROR                  -9
#define BOOTIF_ERROR_INVALID_PROGRAM_TYPE             -10
#define BOOTIF_ERROR_INVALID_INTERRUPT_NUMBER         -11
#define BOOTIF_ERROR_SERVICE_NOT_SUPPORTED            -12
#define BOOTIF_ERROR_BOOTBLOCK_CHECK_SUM_ERROR        -13
#define BOOTIF_ERROR_PROTOCOLFW_CHECK_SUM_ERROR       -14
#define BOOTIF_ERROR_RAM_TEST_READ_UPWARD_ERROR       -15
#define BOOTIF_ERROR_RAM_TEST_READ_DOWNWARD_ERROR     -16
#define BOOTIF_ERROR_DPRAM_TEST_READ_UPWARD_ERROR     -17
#define BOOTIF_ERROR_DPRAM_TEST_READ_DOWNWARD_ERROR   -18
#define BOOTIF_ERROR_ASPC2_REGISTER_ACCESS_ERROR      -19
#define BOOTIF_ERROR_ASPC2_INTERRUPT_ERROR            -20
#define BOOTIF_ERROR_ASPC2_NOT_DETECTED               -21
#define BOOTIF_ERROR_ASPC2_NOT_ONLINE                 -22
#define BOOTIF_ERROR_ASPC2_ACCESS_ERROR               -23
#define BOOTIF_ERROR_UNKNOWN_DEVICE_ID                -24
#define BOOTIF_ERROR_ASPC2_BUS_ACCESS_ERROR           -25

#define TYP_BOOTIF_SIGNAL_ACK_TIME        ((T_TIME_UNIT_1MS) 100)
#define MAX_BOOTIF_SIGNAL_ACK_TIME        ((T_TIME_UNIT_1MS) 2000)

#define TYP_START_FIRMWARE_ACK_TIME       ((T_TIME_UNIT_1MS) 180)
#define MAX_START_FIRMWARE_ACK_TIME       ((T_TIME_UNIT_1MS) 2000)

#define BOOT_BLOCK_EXIST             0x544F4F42uL             /* "BOOT" Bootblock-Exist-Flag */
#define BOOT_BLOCK_VERSION_V200      0x30303256uL             /* "V200" Bootblock-Exist-Flag */
#define BOOT_BLOCK_VERSION_V250      0x30353256uL             /* "V250" Bootblock-Exist-Flag */

#define BOOT_BLOCK_VERSION_V311      0x03110009uL
#define BOOT_BLOCK_VERSION_V300      0x03000002uL                              



#define IN
#define OUT
#define INOUT

#define USIGN8 unsigned char
#define USIGN16 unsigned short
#define T_TIME_UNIT_1MS unsigned short


#define _CONVERT_USIGN16_TO_CONTROLLER_BYTE_ORDERING(Value16)     (Value16)
#define _CONVERT_INT16_TO_CONTROLLER_BYTE_ORDERING(Value16)       (Value16)
#define _CONVERT_USIGN32_TO_CONTROLLER_BYTE_ORDERING(Value32)     (Value32)

#define _CONVERT_USIGN16_TO_HOST_BYTE_ORDERING(Value16)             (_CONVERT_USIGN16_TO_CONTROLLER_BYTE_ORDERING (Value16))
#define _CONVERT_INT16_TO_HOST_BYTE_ORDERING(Value16)               (_CONVERT_INT16_TO_CONTROLLER_BYTE_ORDERING (Value16))
#define _CONVERT_USIGN32_TO_HOST_BYTE_ORDERING(Value32)             (_CONVERT_USIGN32_TO_CONTROLLER_BYTE_ORDERING (Value32))



typedef struct _T_V311_SERVICE_REQ_IF   /* 2060 bytes */
{
  signed short                ReturnCode;

  union
  {
    T_V311_DEVICE_INFO             DeviceInfo;
    T_V311_LICENCE_DATA            LicenceData;
    T_V311_PROGRAM_FLASH           ProgramFlash;
    T_V311_TEST_INTERRUPT          TestInterrupt;
    T_V311_FLASH_CHECKSUM          FlashChecksum;
    T_V311_BUILTIN_SELF_TEST       Bist;

  }                           UseAs;

} PACK_BYTE_ALIGNMENT (T_V311_SERVICE_REQ_IF);


/*=== DEBUG INTERFACE =======================================================*/

typedef struct _T_V311_TRACE
{
  char                        FunctionName [60];

} PACK_BYTE_ALIGNMENT (T_V311_TRACE);

/*---------------------------------------------------------------------------*/

typedef struct _T_V311_DEBUG_LOG
{
  char                        VariableName [32];
  USIGN32               VariableValue;

} PACK_BYTE_ALIGNMENT (T_V311_DEBUG_LOG);

/*---------------------------------------------------------------------------*/

typedef struct _T_V311_DEBUG_IF    /* 96 bytes */
{
  T_V311_TRACE                     Trace;
  T_V311_DEBUG_LOG                 DebugLog;

} PACK_BYTE_ALIGNMENT (T_V311_DEBUG_IF);


/*=== HOST SHARED MEMORY ====================================================*/
#pragma pack(push, 2)
typedef struct _T_V311_HOST_SHARED_MEMORY
{
  T_V311_BOOT_IF_DESCR             BootIfDescr;
  T_V311_SERVICE_REQ_IF            ServiceReq;
  T_V311_DEBUG_IF                  Debug;

} T_V311_BOOT_HOST_SHARED_MEMORY, T_V311_HOST_SHARED_MEMORY; 

//PACK_BYTE_ALIGNMENT (T_V311_HOST_SHARED_MEMORY), PACK_BYTE_ALIGNMENT (T_V311_BOOT_HOST_SHARED_MEMORY);

#pragma pack(pop)

typedef enum _T_BOOT_RESULT
{
  BOOT_OK                                 =  0,

  BOOT_ERR_INIT_HW_FAILED                 = -1,
  BOOT_ERR_REGISTRY_ACCESS_ERROR          = -2,
  BOOT_ERR_FIRMWARE_FILE_NOT_OPENED       = -3,
  BOOT_ERR_BOOTBLOCK_NOT_EXIST            = -4,
  BOOT_ERR_FIRMWARE_FILE_IS_NO_SBN_FILE   = -5,
  BOOT_ERR_NO_CONTROLLER_SIGNAL_ACK       = -6,
  BOOT_ERR_INVALID_SERIAL_NUMBER          = -7,
  BOOT_ERR_EEPROM_WRITE_FAILED            = -8,
  BOOT_ERR_EEPROM_READ_FAILED             = -9,
  BOOT_ERR_INVALID_LENGTH                 = -10,
  BOOT_ERR_INVALID_OFFSET                 = -11,
  BOOT_ERR_FLASH_ERASE_ERROR              = -12,
  BOOT_ERR_FLASH_WRITE_ERROR              = -13,
  BOOT_ERR_READING_FIRMWARE_FILE_FAILED   = -14,
  BOOT_ERR_INVALID_RECORD_LENGTH          = -15,
  BOOT_ERR_CHECK_SUM_ERROR                = -16,
  BOOT_ERR_INVALID_STRING_LENGTH          = -17,
  BOOT_ERR_FLASH_READ_ERROR               = -18,
  BOOT_ERR_INVALID_PROGRAM_TYPE           = -19

} T_BOOT_RESULT;

typedef enum _T_PSAK_RESULT
{
  PSAK_OK                                     = 0,

  /*-------------------------------------------------------------------------*/

  PSAK_NO_EXCEPTION_THROWN,
  PSAK_EXCEPTION_THROWN,

  PSAK_SERVICE_REQ_RES_RESULT_NOT_RECEIVED,
  PSAK_SERVICE_REQ_RES_RESULT_RECEIVED,

  PSAK_NO_SERVICE_CON_RECEIVED,
  PSAK_SERVICE_CON_RECEIVED,

  PSAK_NO_SERVICE_IND_RECEIVED,
  PSAK_SERVICE_IND_RECEIVED,

  PSAK_NO_EVENT_IND_RECEIVED,
  PSAK_EVENT_IND_RECEIVED,
  PSAK_DIAG_HISTORY_NOT_FULL,
  PSAK_DIAG_HISTORY_FULL,

  PSAK_SLAVE_NOT_IN_CYCLIC_DATA_EXCHANGE,
  PSAK_NO_DATA_IND_RECEIVED,
  PSAK_DATA_IND_RECEIVED,

  /*-------------------------------------------------------------------------*/

  PSAK_ERR_FATAL_ERROR                        = -1,

  PSAK_ERR_INVALID_CHANNEL_ID                 = -2,
  PSAK_ERR_CHANNEL_ALREADY_IN_USE             = -3,
  PSAK_ERR_CHANNEL_NOT_IN_USE                 = -4,
  PSAK_ERR_NO_PROTOCOL_SOFTWARE_RESPONSE      = -5,
  PSAK_ERR_INVALID_NUMBER_ARRAY_ELEMENTS      = -6,
  PSAK_ERR_INVALID_POINTER_VALUE              = -7,
  PSAK_ERR_SERVICE_IF_ALREADY_BUSY            = -8,
  PSAK_ERR_INVALID_SLAVE_ADDRESS              = -9,
  PSAK_ERR_INVALID_DATA_SIZE                  = -10,
  PSAK_ERR_INVALID_DATA_OFFSET                = -11,
  PSAK_ERR_MEM_ALLOC_FAIL                     = -12,
  PSAK_ERR_INTERFACE_VERSION_CONFLICT         = -13,
  PSAK_ERR_EVENT_IF_ALREADY_BUSY              = -18,
  PSAK_ERR_MODE_DOES_NOT_SUPPORT_FUNCTION     = -19,


  PSAK_ERR_START_PENDING                      = -50,

} T_PSAK_RESULT;

#pragma pack(push,1)
typedef union _T_SBN_HEADER_DESCR
{
  struct _T_SBN_HEADER
  {
    unsigned short  Type;
    __s32           Address;
    unsigned short  Length;

  } __attribute__ ((packed))  Struct;

  unsigned char Raw [sizeof (struct _T_SBN_HEADER)];

} __attribute__ ((packed)) T_SBN_HEADER_DESCR;
#pragma pack(pop)


/******************************************************************************
GLOBAL DATA
******************************************************************************/

// extern T_V311_HOST_SHARED_MEMORY    FAR HostSharedMemory;


/*****************************************************************************/

#endif /* __BOOTV311_H__ */
