/*****************************************************************************/
/*                                                                           */
/*                              SOFTING AG                                   */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*                  Copyright (C) SOFTING AG 1995-2007                       */
/*                                                                           */
/* See file CREDITS for list of people who contributed to this               */
/* project.                                                                  */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    */
/* MA 02111-1307, USA.                                                       */
/*                                                                           */
/* email: support.automation@softing.com                                     */
/* Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany                 */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Author      : SOFTING                                                    */
/*                                                                           */
/*  CHANGE_NOTES                                                             */
/*                                                                           */
/*  date      name      change                                               */
/*  -----------------------------------------------------------------------  */
/*****************************************************************************/
/*****************************************************************************/

/***************************************************************************/
/*                                                                         */
/*          LINUX PROFIBUS DEVICE DRIVER FOR SOFTING PROFIBOARD            */
/*                                                                         */
/* The data structure CtrlExt has entries for 4 possible boards. For each  */
/* board the hw-resources are stored. In addition this data structure      */
/* contains sub structures for (direct) pointers to the DP-Ram memory      */
/* of the different services (for example start address of DP input        */
/* and output areas).                                                      */
/*                                                                         */
/* For DP two bottom half interrupt service threads exist. Only one        */
/* of them is used, they are switched - if necessary - in the ISR.         */
/*                                                                         */
/* The device driver uses _NOT_ a dynamic major number. A fixed value has  */
/* been choosen (122). It has been written, in mind, that several boards   */
/* have to be supported (later) and that it can be ported to other         */
/* hardware platforms.                                                     */
/*                                                                         */
/* Data device: The driver offers two access modes to the data device:     */
/*              - the normal mode and the                                  */
/*              - optimized mode.                                          */
/*              In the NORMAL mode a read call to the data device returns  */
/*              immideatly with the requested data.                        */
/*              In the OPTIMIZED mode a read call to the data device       */
/*              will only return immideatly, if the data image had been    */
/*              changed since the last successful read call. In case the   */
/*              device had been opened in blocking mode, the calling       */
/*              thread/process will be blocked, until real new data is     */
/*              available, in case of non-blocking mode the function       */
/*              returns immideatly but without any data (count=0).         */
/*              The OPTIMIZED mode can only be enabled globally. This      */
/*              means, that if the data device is opened several times     */
/*              and via one file descriptor the device is set to the       */
/*              OPTIMIZED mode, this mode is used for every fd!            */
/*                                                                         */
/* init_module:                                                            */
/*   - searching of HW (board)                                             */
/*   - allocation of HW-resources                                          */
/*   - initializing of CtrlExt                                             */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/* ----                                                                    */
/* A good resource for information about writing Linux-Device-Drivers is:  */
/* Alessandro Rubini: LINUX Device Drivers, O'Reilly & Associates, INC,    */
/*                    February 1998                                        */
/***************************************************************************/

#ifdef PB_PCI_SUPPORT  


#if defined(PB_ISA_SUPPORT) && defined (PB_PCI_SUPPORT)
#error this driver does not support mixing of isa and pci boards
#endif

#include <linux/module.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33)
#include <linux/autoconf.h>
#endif
#include <asm/io.h>
#include <asm/segment.h>


#include <linux/pci.h>
#include "kernel_version.h"

#include <linux/poll.h>


#include <linux/slab.h>
#include <linux/proc_fs.h>

#include "basetypes.h"

#include "pb_debug.h"
#include "cntrl.h"
#include "cmi.h"
#include "xx_if.h"
#include "xx_err.h"
#include "pb_xxx.h"
#include "fw.h"
#include "pb_ioctl.h"
#include "pb_hw_pci.h"




#if defined(CONFIG_MODVERSION) && !defined(MODVERSION)
#define MODVERSIONS
#endif
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif

#ifdef DEBUG
#undef MOD_DEC_USE_COUNT
#undef MOD_INC_USE_COUNT

#define MOD_DEC_USE_COUNT
#define MOD_INC_USE_COUNT
#endif

#if (VERSION_CODE(2,4,0) <= LINUX_VERSION_CODE)
MODULE_LICENSE ("GPL");
#endif

#define PB_STATISTICS

#define PB_IO_START 0x200
#define PB_IO_END   0x400
#define PB_IO_RANGE 0x04

#define PB_HW_CONFIG_TIMEOUT  1 * HZ/100


extern void DumpMem(void *pMem, unsigned long size, char *pText);
extern int cmi_init( struct CntrlExt *CtrlExt );


VOID hw_pci_remove_all(struct pci_dev * PciDev);
int hw_pci_probe_all (struct pci_dev *PciDev, const struct pci_device_id *PciId);
VOID hw_pci_remove(struct pci_dev * PciDev);
int hw_pci_probe (struct pci_dev *PciDev, const struct pci_device_id *PciId);
int hw_pci_pro_probe (struct pci_dev *PciDev, const struct pci_device_id *PciId);
int hw_pcie_probe (struct pci_dev *PciDev, const struct pci_device_id *PciId);
void PbStartCntrlPcie( struct CntrlExt *Board );
void PbStopCntrlPcie( struct CntrlExt *Board );

void PbDataISTFirst( TASKPARATYPE Args );
void PbDataIST( TASKPARATYPE Args );
void PbSlaveIST( TASKPARATYPE Args );

#ifndef irqreturn_t 
//#define irqreturn_t IRQRETTYPE
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
IRQRETTYPE profibus_irq( int irq, void *dev_id, struct pt_regs *regs );
#else
    irqreturn_t profibus_irq( int irq, void *dev_id );
#endif
void await_irq_reset( struct CntrlExt *CtrlExt);



extern struct file_operations PrFileOps;

extern char *Version;
extern char *Copyright;
extern char *Warranty;
extern struct semaphore PbGlobalSema;

spinlock_t PlxSpinlock;


struct pci_device_id HwPciIdTable[] = 
{
  {
    PLX_VENDOR_ID,
    PLX_9050_DEVICE_ID,
    SOFTING_VENDOR_ID,
    SOFTING_PB_PCI_DEVICE,
    0,
    0,
    0
  },
  
  {
    PLX_VENDOR_ID,
    PLX_9030_DEVICE_ID,
    SOFTING_VENDOR_ID,
    SOFTING_PB_PRO_DEVICE,
    0,
    0,
    0
  },
  {
    PLX_VENDOR_ID,
    PLX_9030_DEVICE_ID,
    SOFTING_VENDOR_ID,
    SOFTING_PB_PRO_104_DEVICE,
    0,
    0,
    0
  },
  {
    PLX_VENDOR_ID,
    PLX_86E1_DEVICE_ID,
    SOFTING_VENDOR_ID,
    SOFTING_PB_PCIE_DEVICE,
    0,
    0,
    0
  },
  
   {0, 0, 0, 0, 0, 0, 0}
};


/*---------------------------------------------------------------------------*/
// PCI Driver Structure for hotplugging.

struct pci_driver HwPci = 
{
  name:PB_NAME,
  probe:hw_pci_probe_all,
  remove:hw_pci_remove_all,
  id_table:HwPciIdTable
};

static unsigned int       major_number;

u32 getPciOperationModeForAsic(struct CntrlExt *CtrlExt, u32 Channel)
{
  USIGN32            i;
  u8 *            pPciCfgRegBaseAddr;          // mapped local configuration registers base address
  u32 volatile * pPciCfgRegCNTRL;             // local configuration CNTRL register
  u32            data = 0;
  u32            hdata = 0;
  u32            PciCfgRegCNTRL;

  
  if (
      (CtrlExt->DeviceType == SOFTING_PB_PRO_DEVICE) ||
      (CtrlExt->DeviceType == SOFTING_PB_PCIE_DEVICE) ||
      (CtrlExt->DeviceType == SOFTING_PB_PRO_104_DEVICE) 
     )
      
  {
    return(ASIC_ASPC2_SPC3);
  }


   // map configuration registers memory


   // virtual address - cache disabled
  pPciCfgRegBaseAddr = ioremap_nocache(CtrlExt->config_space_mem, 0x80); 
    //MmMapIoSpace(PhysAddr,CtrlExt->config.PciCfgRegBaseSize,FALSE);
   
  pPciCfgRegCNTRL    = (u32*) (pPciCfgRegBaseAddr + CONTROL_REGISTER);
  PciCfgRegCNTRL     = *pPciCfgRegCNTRL;

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;     // set CS
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x06000000;     // set Startbit
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x07000000;
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x06000000;     // set OpCode for Read
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x07000000;
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x03000000;
  udelay(3);
   


   // set offset in PCI Controll Register (hex 32)

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x06000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x07000000;     // 1
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x06000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x07000000;     // 1
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x03000000;     // 0
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x03000000;     // 0
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x06000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x07000000;     // 1
  udelay(3);
   

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x03000000;     // 0
  udelay(3);
   


  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;     // read dummy bit
  udelay(3);
   
  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x03000000;
  udelay(3);
   


  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;
  udelay(3);
   
   // read value
  for (i=0; i<=15; i++)
  {
    if ((*pPciCfgRegCNTRL & GET_DO)) 
    {
      hdata = 1;
    }
    else 
    {
      hdata = 0;
    }
      
    *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x03000000;
    udelay(3);
      
    *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x02000000;
    udelay(3);
      
    data = data << 1;
    data |= hdata;
  }

  *pPciCfgRegCNTRL = PciCfgRegCNTRL | 0x00000000;
  *pPciCfgRegCNTRL = PciCfgRegCNTRL;

   //MmUnmapIoSpace(pPciCfgRegBaseAddr,CtrlExt->config.PciCfgRegBaseSize);
  io_unmap ((void*) pPciCfgRegBaseAddr);
   
  switch (Channel)
  {
    case 0:
      return(data & 0x000000FF);         // return ASIC-Types for channel 0

    case 1:
      return((data & 0x0000FF00) >> 8);  // return ASIC-Types for channel 1

    default:
      return(0x0L);        // no valid channel return no ASIC Types
  }
}


void PbStartCntrlPci( struct CntrlExt *Board )
{
  unsigned long flags;
  long Cntrl;
  
  GetSpinlock(Board, &flags, __LINE__);
  
  Cntrl  = inl ((Board->ConfigBase + CONTROL_REGISTER));
  
  
  switch (Board->Channel)
  {
    case 0:
    {
      Cntrl |= RUN_CHANNEL_0;
      break;
    }
    case 1:
    {
      Cntrl |= RUN_CHANNEL_1;
      break;
    }
  } /* switch (Board->ChannelNumber) */
  
  outl (Cntrl, (Board->ConfigBase + CONTROL_REGISTER));
  ReleaseSpinlock(Board, flags, __LINE__);
  
}

void PbStartCntrlPcie( struct CntrlExt *Board )
{
//   unsigned long flags;
  USIGN16 *pReset = (USIGN16*)(Board->DprAddr + 0x7FFF8);
  
  *pReset = 1;
}

void PbStopCntrlPcie( struct CntrlExt *Board )
{
//   unsigned long flags;
  USIGN16 *pHalt = (USIGN16*)(Board->DprAddr + 0x7FFFA);
  
  *pHalt = 1;
}

void PbStartCntrlProPci( struct CntrlExt *Board )
{
  unsigned long flags;
  long Cntrl;
  long PlxCntrl;

  GetSpinlock(Board, &flags, __LINE__);
  
  PlxCntrl  = inl ((Board->ConfigBase + CONTROL_REGISTER));
  Cntrl  = inl ((Board->ConfigBase + GPIOC_9030));
  
 
  switch (Board->Channel)
  {
    case CHANNEL0:
    {
      Cntrl |= RUN_CHANNEL_0;
      break;
    }
    case CHANNEL1:
    {
      Cntrl |= RUN_CHANNEL_1;
      break;
    }
  } /* switch (Board->ChannelNumber) */

  PlxCntrl |= 0x80054000;                    // patch PLX control register

  outl (PlxCntrl, (Board->ConfigBase + CONTROL_REGISTER));
  outl (Cntrl, (Board->ConfigBase + GPIOC_9030));
  
  
  ReleaseSpinlock(Board, flags, __LINE__);
 

}

void PbStartCntrl( struct CntrlExt *Board )
{
  
  switch(Board->DeviceType)
  {
    case SOFTING_PB_PRO_DEVICE:
    case SOFTING_PB_PRO_104_DEVICE:
      PbStartCntrlProPci(Board);
      break;    
    
    case SOFTING_PB_PCIE_DEVICE:
      PbStartCntrlPcie(Board);
      break;
      
    case SOFTING_PB_PCI_DEVICE:
      PbStartCntrlPci(Board);
      break;
  }
  
}


void PbStopChannel(struct CntrlExt *Board)
{
  long Cntrl;
  
  switch(Board->DeviceType)
  {
    case SOFTING_PB_PRO_DEVICE:
    case SOFTING_PB_PRO_104_DEVICE:
      Cntrl  = inl ((Board->ConfigBase + GPIOC_9030));
      switch (Board->Channel)
      {
        case CHANNEL0:
        {
          Cntrl &= RESET_CHANNEL_0;
          break;
        }
        case CHANNEL1:
        {
          Cntrl &= RESET_CHANNEL_1;
          break;
        }
      } /* switch (Board->ChannelNumber) */
      outl (Cntrl, (Board->ConfigBase + GPIOC_9030));    
      break;    
      
    case SOFTING_PB_PCIE_DEVICE:
      {
	USIGN16 *pHalt = (USIGN16*)(Board->DprAddr + 0x7FFF8);
	*pHalt = 1;
      }
    break;
      
    case SOFTING_PB_PCI_DEVICE:
      Cntrl  = inl ((Board->ConfigBase + CONTROL_REGISTER));
      switch (Board->Channel)
      {
	case 0:
	{
	  Cntrl &= RESET_CHANNEL_0;
	  break;
	}
	case 1:
	{
	  Cntrl &= RESET_CHANNEL_1;
	  break;
	}
      } /* switch (Board->ChannelNumber) */
      outl (Cntrl, (Board->ConfigBase + CONTROL_REGISTER));
      break;
  }
}


int PbInitPciBoard( struct CntrlExt *Board, char Reset )
{
  
  long Cntrl;
  long IntCsr;

 // printk("PbInitBoard(0x%lx, %u)\n", (unsigned long)Board, Reset);
  
  if (Board->ConfigBase)
  {
    Cntrl  = inl ((Board->ConfigBase + CONTROL_REGISTER));
    IntCsr = inl ((Board->ConfigBase + IRQ_REGISTER));
 
    switch (Board->Channel)
    {
      case 0:
      {
        if (Reset == C165_RESET) 
        { 
          Cntrl |= RUN_CHANNEL_0; 
          IntCsr |= ENABLE_IRQ_CHANNEL_0; 
        }
        else                     
        { 
          Cntrl &= RESET_CHANNEL_0; 
          IntCsr &= DISABLE_IRQ_CHANNEL_0; 
        }

        break;
      }
      case 1:
      {
        if (Reset == C165_RESET) 
        { 
          Cntrl |= RUN_CHANNEL_1; 
          IntCsr |= ENABLE_IRQ_CHANNEL_1; 
        }
        else                     
        { 
          Cntrl &= RESET_CHANNEL_1; 
          IntCsr &= DISABLE_IRQ_CHANNEL_1; 
        }

        break;
      }
    } /* switch (Board->ChannelNumber) */

 
    outl (IntCsr, (Board->ConfigBase + IRQ_REGISTER));
    outl (Cntrl, (Board->ConfigBase + CONTROL_REGISTER));

    // XXX - Autsch, busy loop!
    udelay( 35 ); // IRQ has to be activated for at least 25 usec
  }


  return( 0 );
}

int PbInitPcieBoard( struct CntrlExt *Board, char Reset )
{
  
  long IntCsr = 0;
  
  // printk("PbInitBoard(0x%lx, %u)\n", (unsigned long)Board, Reset);
  
  if (Board->ConfigBase)
  {
    IntCsr = inl ((Board->ConfigBase + _86E1_INTCSR));
    
    if (Reset != C165_RESET) 
    { 
      PbStartCntrlPcie(Board); 
      IntCsr |= _86E1_EN_IRQ; 
    }
    else                     
    { 
      PbStopCntrlPcie(Board); 
      IntCsr &= _86E1_EN_IRQ; 
    }
    
    outl (IntCsr, (Board->ConfigBase + _86E1_INTCSR));
    
    // XXX - Autsch, busy loop!
    udelay( 35 ); // IRQ has to be activated for at least 25 usec
  }
  
  
  return( 0 );
}


int PbInitProBoard( struct CntrlExt *Board, char Reset )
{
  
  long Cntrl;
  long IntCsr;
  long PlxCntrl;

  printk("PbInitProBoard(0x%lx, %u)\n", (unsigned long)Board, Reset);
  
  if (Board->ConfigBase)
  {
    Cntrl  = inl ((Board->ConfigBase + GPIOC_9030));
    IntCsr = inl ((Board->ConfigBase + IRQ_REGISTER));
    PlxCntrl = inl ((Board->ConfigBase + CONTROL_REGISTER));
    
   printk("PbInitProBoard: Cntrl=0x%lx, IntCsr=0x%lx\n",Cntrl,IntCsr);

    switch (Board->Channel)
    {
      case CHANNEL0:
      {
        if (Reset == C165_RESET) 
        { 
          Cntrl |= RUN_CHANNEL_0; 
          IntCsr |= ENABLE_IRQ_CHANNEL_0; 
        }
        else                     
        { 
          Cntrl &= RESET_CHANNEL_0; 
          IntCsr &= DISABLE_IRQ_CHANNEL_0; 
        }

        break;
      }
      case CHANNEL1:
      {
        if (Reset == C165_RESET) 
        { 
          Cntrl |= RUN_CHANNEL_1; 
          IntCsr |= ENABLE_IRQ_CHANNEL_1; 
        }
        else                     
        { 
          Cntrl &= RESET_CHANNEL_1; 
          IntCsr &= DISABLE_IRQ_CHANNEL_1; 
        }

        break;
      }
    } /* switch (Board->ChannelNumber) */

    PlxCntrl |= 0x80054000;                    // patch PLX control register

    outl (PlxCntrl, (Board->ConfigBase + CONTROL_REGISTER));
    outl (IntCsr, (Board->ConfigBase + IRQ_REGISTER));
    outl (Cntrl, (Board->ConfigBase + GPIOC_9030));

    // XXX - Autsch, busy loop!
    udelay( 35 ); // IRQ has to be activated for at least 25 usec
  }


  return( 0 );
}

int PbInitBoard( struct CntrlExt *Board, char Reset )
{
  switch(Board->DeviceType)
  {
    case SOFTING_PB_PCIE_DEVICE:
      return PbInitPcieBoard(Board,Reset);
      break;
      
     case SOFTING_PB_PRO_DEVICE:
     case SOFTING_PB_PRO_104_DEVICE:
      return PbInitProBoard(Board,Reset);
      break;
      
     case SOFTING_PB_PCI_DEVICE:
      return PbInitPciBoard(Board,Reset);
      break;
  }
  
  return -1;
}


void irq_to_pci_cntrl( struct CntrlExt *CtrlExt, unsigned char IrqValue)
{
  short volatile *h_to_c_irq_val; 
  unsigned long irqflags;

  long Cntrl; // = inl ((CtrlExt->ConfigBase + CONTROL_REGISTER));
  
  await_irq_reset( CtrlExt );
  
  spin_lock_irqsave(&PlxSpinlock, irqflags);
  
  Cntrl = inl ((CtrlExt->ConfigBase + CONTROL_REGISTER));
  mb ();

  h_to_c_irq_val = CtrlExt->h_to_c_irq_val; //(char *)CtrlExt->DprAddr + H_TO_C_IRQ_VAL_OFFSET;

  if( IrqValue == 0 )
    printk(KERN_ERR "illegal Interrupt: irq_to_cntrl( %p, %d)\n",
           CtrlExt, IrqValue );

  write_io_word( h_to_c_irq_val, IrqValue); // set IRQ identifier


  switch (CtrlExt->Channel)
  {
    case 0: Cntrl &= CLEAR_IRQ_CHANNEL_0; break;
    case 1: Cntrl &= CLEAR_IRQ_CHANNEL_1; break;
  } /* switch (pFile->Channel) */

  outl (Cntrl, (CtrlExt->ConfigBase + CONTROL_REGISTER));
  wmb ();

  switch (CtrlExt->Channel)
  {
    case 0: Cntrl |= SET_IRQ_CHANNEL_0; break;
    case 1: Cntrl |= SET_IRQ_CHANNEL_1; break;
  } /* switch (pFile->Channel) */

  outl (Cntrl, (CtrlExt->ConfigBase + CONTROL_REGISTER));
  wmb ();

  spin_unlock_irqrestore(&PlxSpinlock, irqflags);

  PR_DEBUG(D_OTHER_L3,"profiboard(irq_to_cntrl): %d h2c: 0x%X\n", IrqValue,
           read_io_byte( h_to_c_irq_val ) );
  return;
}

void disable_pcie_irq(struct CntrlExt *CtrlExt)
{
  long IntCsr = 0;
  
  IntCsr = inl ((CtrlExt->ConfigBase + _86E1_INTCSR));
  IntCsr &= _86E1_EN_IRQ; // disable irq_to_cntrl
  outl (IntCsr, (CtrlExt->ConfigBase + _86E1_INTCSR));
  
  return;
}

void irq_to_pcie_cntrl( struct CntrlExt *CtrlExt, unsigned char IrqValue)
{
  short          volatile *h_to_c_irq_val;
  unsigned short volatile irqVal = IrqValue;
//   unsigned long irqflags;
  volatile int test = 0;
  
  await_irq_reset( CtrlExt );
  
  h_to_c_irq_val = CtrlExt->h_to_c_irq_val; 
  
  if( irqVal == 0 )
    printk(KERN_ERR "illegal Interrupt: irq_to_cntrl( %p, %d)\n",
	   CtrlExt, irqVal );
    *h_to_c_irq_val = (USIGN16) irqVal;          /* set IRQ identifier and trigger IRQ */
    while (*h_to_c_irq_val != (USIGN16) irqVal)  /* wait till IrqValue is set */
    {
      test++;
      printk("irq_to_pcie_cntrl: wrong value!0x%x!=0x%x\n",*h_to_c_irq_val,irqVal);
      if(test>3)
      {
	disable_pcie_irq(CtrlExt);
	return;
      }
    }
    
  PR_DEBUG(D_OTHER_L3,"profiboard(irq_to_cntrl): %d h2c: 0x%X\n", irqVal,
	   read_io_byte( h_to_c_irq_val ) );
  return;
}
    
    
void irq_to_pro_cntrl( struct CntrlExt *CtrlExt, unsigned char IrqValue)
{
  short          volatile *h_to_c_irq_val;
  unsigned short volatile irqVal = IrqValue;
  unsigned long irqflags;

  long Cntrl;  
  
 // printk("irq_to_pro_cntrl(0x%lx) 0x%x\n",CtrlExt->DprAddr,irqVal);

  await_irq_reset( CtrlExt );
  
//   printk("spin_lock_irq\n");
//   spin_lock_irq(&PlxSpinlock);
  spin_lock_irqsave(&PlxSpinlock, irqflags);
  mb ();
  Cntrl = inl ((CtrlExt->ConfigBase + GPIOC_9030));
  
  h_to_c_irq_val = CtrlExt->h_to_c_irq_val; //(char *)CtrlExt->DprAddr + H_TO_C_IRQ_VAL_OFFSET;

  if( irqVal == 0 )
    printk(KERN_ERR "illegal Interrupt: irq_to_cntrl( %p, %d)\n",
           CtrlExt, irqVal );

  write_io_word( h_to_c_irq_val, irqVal);              // set IRQ identifier
  while (read_io_word(h_to_c_irq_val) != irqVal)       // read IRQ identfier
  {
    write_io_word( h_to_c_irq_val, irqVal);            // set IRQ identifier
  }

  switch (CtrlExt->Channel)
  {
    case CHANNEL0: Cntrl &= CLEAR_IRQ_CHANNEL_0; break;
    case CHANNEL1: Cntrl &= CLEAR_IRQ_CHANNEL_1; break;
  } /* switch (pFile->Channel) */

  outl (Cntrl, (CtrlExt->ConfigBase + GPIOC_9030));
  wmb ();

  switch (CtrlExt->Channel)
  {
    case CHANNEL0: Cntrl |= SET_IRQ_CHANNEL_0; break;
    case CHANNEL1: Cntrl |= SET_IRQ_CHANNEL_1; break;
  } /* switch (pFile->Channel) */

  outl (Cntrl, (CtrlExt->ConfigBase + GPIOC_9030));
  wmb ();

  spin_unlock_irqrestore(&PlxSpinlock, irqflags);
  
//   spin_unlock_irq(&PlxSpinlock);

//   printk("spin_unlock_irq\n");

  PR_DEBUG(D_OTHER_L3,"profiboard(irq_to_cntrl): %d h2c: 0x%X\n", irqVal,
           read_io_byte( h_to_c_irq_val ) );
  return;
}

void irq_to_cntrl(struct CntrlExt *CtrlExt, unsigned char IrqValue)
{
  switch(CtrlExt->DeviceType)
  {   
    case SOFTING_PB_PRO_DEVICE:
    case SOFTING_PB_PRO_104_DEVICE:
      irq_to_pro_cntrl(CtrlExt,IrqValue);
      break;
      
    case SOFTING_PB_PCIE_DEVICE:
      irq_to_pcie_cntrl(CtrlExt,IrqValue);
      break;
      
     case SOFTING_PB_PCI_DEVICE:
      irq_to_pci_cntrl(CtrlExt,IrqValue);
      break;
       
    default:
      printk("profiboard: invalid device!\n");
      break;
      
  }
}

int PbStartBoard( struct CntrlExt *Board, char Reset )
{
  char version_string [VERSION_STRING_LENGTH];

  if ( (Board->port == 0) && (Board->ConfigBase == 0) ) 
  {
    return( -1 );
  }

  PbInitBoard( Board, Reset );
  if( cmi_init( Board ) < 0 ) 
  {
    printk("profiboard: CONTROLLER NOT RESPONDING\n");
    DumpMem((char*)Board->DprAddr, 0x100, "DPRAM");
    return( -1 );
  }
  Board->Service.CMIClosed = 0;
  // check board type by reading the version string at offset 0x60
  // The board needs a few micro-seconds to write the magic strings into the
  // DPRAM.
  Board->TypeOfFW = PB_MASTER_FW;
  copy_from_io (version_string,
                (char*) (Board->DprAddr + PB_VERSION_OFFSET),
                sizeof (version_string));

  //printk("PbStartBoard: version=%s\n",version_string);
  
  if (strstr (version_string, "PROFI")) 
  {
    PR_DEBUG(D_ALL_L1, "now checking type ...");
    if (strstr (version_string, "DPS") || strstr (version_string, "DP-Slave")) 
    {
      PR_DEBUG(D_ALL_L1, "DP-Slave software\n");
      Board->TypeOfFW = PB_DP_SLAVE_FW;
    } 
    else if (strstr (version_string, "FMS")) 
    {
      PR_DEBUG(D_ALL_L1, "Master-Software\n");
      Board->TypeOfFW = PB_MASTER_FW;
    } 
    else 
    {
      printk("Firmware type (master/slave) not identified!\n");
    }
  }
  else
  {
    
    PR_DEBUG(D_ALL_L1, "now checking type ...");
    if (strstr (version_string, "DP-Slave")) 
    {
      PR_DEBUG(D_ALL_L1, "DP-Slave software\n");
      Board->TypeOfFW = PB_DP_SLAVE_FW;
    } 
    else if (strstr (version_string, "DP(V1)/FDL")) 
    {
      PR_DEBUG(D_ALL_L1, "Master-Software\n");
      Board->TypeOfFW = PB_MASTER_FW;
    } 
    else if (strstr (version_string, "FMS")) 
    {
      PR_DEBUG(D_ALL_L1, "Master-Software\n");
      Board->TypeOfFW = PB_MASTER_FW;
    } 
    else 
    {
      printk("Firmware type (master/slave) not identified!\n");
    }
    
  }

  return( 0 );
}

/******************/
/* Module Section */
/******************/

static inline char* pcibios_strerror (int error_code)
{
  switch (error_code)
  {
    case PCIBIOS_SUCCESSFUL         : return ("ok");
    case PCIBIOS_FUNC_NOT_SUPPORTED : return ("function not supported");
    case PCIBIOS_BAD_VENDOR_ID      : return ("bad vendor id");
    case PCIBIOS_DEVICE_NOT_FOUND   : return ("device not found");
    case PCIBIOS_BAD_REGISTER_NUMBER: return ("bad register number");
    case PCIBIOS_SET_FAILED         : return ("set failed");
    case PCIBIOS_BUFFER_TOO_SMALL   : return ("buffer too small");
    default                         : return ("unknown error code");
  } /* switch (error_code) */
}

int initProfiPciDriver(void)
{
  
  int result = 0;
  
  
  printk("initProfiPciDriver:\n");
 // SET_MODULE_OWNER(&PrFileOps);
  major_number = register_chrdev(PB_MAJOR, PB_NAME, &PrFileOps);
  
  if (major_number >= 0)
  {
    
    result = 0;
    
    printk("initProfiPciDriver: major = %u\n", major_number);
  }
  else
  {
    printk("initProfiPciDriver:can't get major number\n");
    return -EBUSY;
  }
  
  if(pci_register_driver(&HwPci) < 0)
  {
    printk("initProfiPciDriver: (HwPci) failed!\n");
    return -EIO;
  }
  
   
  return result;
}

VOID hw_pci_remove_all(struct pci_dev * PciDev)
{

  hw_pci_remove(PciDev);
  
}

/*****************************************************************************/
/* hw_pci_remove                                                             */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   this function will be called when a device is removed                   */
/*   from the bus which matches one of the device tables defined below       */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*    struct pci_dev       : contains the resource information for the       */
/*                           device                                          */
/*                                                                           */
/*****************************************************************************/

VOID hw_pci_remove(struct pci_dev * PciDev)
{
    
  int Cntrl, BoardIndex, BoardIndex2;
  unsigned long flags;

  printk("hw_pci_remove:\n");
  
  
  
  for(BoardIndex=0; 
      ((BoardIndex<PB_MAX_NR_BOARDS) && (CtrlExt [BoardIndex].physicalDPRAM != pci_resource_start(PciDev, 2))); 
      BoardIndex++)
  {
    ;
  }
  
  printk("hw_pci_remove: board %u\n",BoardIndex);
  
  if(BoardIndex >= PB_MAX_NR_BOARDS) // no such device!
    return;
  
  if(!CtrlExt [BoardIndex].isUsed) // no such device!
    return;
  
  if(CtrlExt->HSdbBuffer)
  {
    kfree(CtrlExt->HSdbBuffer);
    CtrlExt->HSdbBuffer = NULL;
  }
  
  CtrlExt [BoardIndex].isUsed = FALSE;
  CtrlExt [BoardIndex].physicalDPRAM = 0;
  
  tasklet_kill(&CtrlExt [BoardIndex].PbSlaveTasklet);
  tasklet_kill(&CtrlExt [BoardIndex].PbFirstTasklet);
  tasklet_kill(&CtrlExt [BoardIndex].PbDataTasklet);

  
  if(CtrlExt [BoardIndex].ConfigBase)
  {
   // save_flags(flags);
   // cli(); solved 21.2.08 kno
    GetSpinlock(&CtrlExt[BoardIndex], &flags, __LINE__);
    Cntrl  = inl ((CtrlExt [BoardIndex].ConfigBase + CONTROL_REGISTER));
    ReleaseSpinlock(&CtrlExt[BoardIndex], flags, __LINE__);
   // restore_flags (flags);

    switch ( CtrlExt [BoardIndex].Channel)
    {
      case CHANNEL0: Cntrl &= RESET_CHANNEL_0; break;
      case CHANNEL1: Cntrl &= RESET_CHANNEL_1; break;
    } /* switch (Board->ChannelNumber) */

   // save_flags(flags);
   // cli(); solved 21.2.08 kno
    GetSpinlock(&CtrlExt[BoardIndex], &flags, __LINE__);
    outl (Cntrl, (CtrlExt [BoardIndex].ConfigBase + CONTROL_REGISTER));
    ReleaseSpinlock(&CtrlExt[BoardIndex], flags, __LINE__);
   // restore_flags (flags);
  }

  if(CtrlExt [BoardIndex].interrupt)
  {
    free_irq( CtrlExt [BoardIndex].interrupt, &CtrlExt[BoardIndex] );
    CtrlExt [BoardIndex].interrupt = 0;
  }
  
  
  if(CtrlExt [BoardIndex].DprAddr)
  {
    io_unmap ((void*) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].DprAddr = 0;
   
  }
  
  
  if( CtrlExt[BoardIndex].FWVersion ) 
  {
    kfree( CtrlExt[BoardIndex].FWVersion );
    CtrlExt[BoardIndex].FWVersion = NULL;
  }
  
 
 
  if(!pci_resource_start(PciDev, 3)) // single channel!
  {
    if(CtrlExt [BoardIndex].ConfigBase)
    {
      release_region (CtrlExt [BoardIndex].ConfigBase, pci_resource_len(PciDev, 1));
      CtrlExt [BoardIndex].ConfigBase = 0;
    }
    printk("hw_pci_remove: done at line %u \n",__LINE__);
    return; 
  }
  
  BoardIndex2 = BoardIndex;
   //++BoardIndex;
  for(BoardIndex=0; 
      ((BoardIndex<PB_MAX_NR_BOARDS) && (CtrlExt [BoardIndex].physicalDPRAM != pci_resource_start(PciDev, 3))); 
      BoardIndex++)
  {
    ;
  }

  if(BoardIndex >= PB_MAX_NR_BOARDS) // no such device!
  {
    if(CtrlExt [BoardIndex2].ConfigBase)
    {
      release_region (CtrlExt [BoardIndex2].ConfigBase, pci_resource_len(PciDev, 1));
      CtrlExt [BoardIndex2].ConfigBase = 0;
    }
    printk("hw_pci_remove: done at line %u \n",__LINE__);
    return;
  }

  
  if(!CtrlExt [BoardIndex].isUsed) // no such device!
  {
    if(CtrlExt [BoardIndex2].ConfigBase)
    {
      release_region (CtrlExt [BoardIndex2].ConfigBase, pci_resource_len(PciDev, 1));
      CtrlExt [BoardIndex2].ConfigBase = 0;
    }
    printk("hw_pci_remove: done at line %u \n",__LINE__);
    return;
  }
  


  CtrlExt [BoardIndex].isUsed = FALSE;
  tasklet_kill(&CtrlExt [BoardIndex].PbSlaveTasklet);
  tasklet_kill(&CtrlExt [BoardIndex].PbFirstTasklet);
  tasklet_kill(&CtrlExt [BoardIndex].PbDataTasklet);

  
  if(CtrlExt [BoardIndex].ConfigBase)
  {
    GetSpinlock(&CtrlExt[BoardIndex], &flags, __LINE__);
    PbStopChannel(&CtrlExt [BoardIndex]);
    ReleaseSpinlock(&CtrlExt[BoardIndex], flags, __LINE__);
  }
  

  if(CtrlExt [BoardIndex].DprAddr)
  {
    printk("free DPRAM memory\n");
    io_unmap ((void*) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].DprAddr = 0;
  }

  if(CtrlExt [BoardIndex].ConfigBase)
  {  
    printk("free config space\n");
    release_region (CtrlExt [BoardIndex].ConfigBase, pci_resource_len(PciDev, 1));
    CtrlExt [BoardIndex].ConfigBase = 0;
  }
  
  if(CtrlExt [BoardIndex].interrupt)
  {
    printk("free interrupt\n");
    free_irq( CtrlExt [BoardIndex].interrupt, &CtrlExt[BoardIndex] );
    CtrlExt [BoardIndex].interrupt = 0;
  }
  
  
  if( CtrlExt[BoardIndex].FWVersion ) 
  {
    printk("free memory\n");
    kfree( CtrlExt[BoardIndex].FWVersion );
    CtrlExt[BoardIndex].FWVersion = NULL;
  }
   
  printk("hw_pci_remove: done at line %u \n",__LINE__);  
  return;
  
}

int hw_pci_probe_all(struct pci_dev *PciDev, const struct pci_device_id *PciId)
{
  
  switch(PciId->subdevice)
  {
    case SOFTING_PB_PCI_DEVICE:
      return hw_pci_probe(PciDev, PciId);
      break;
      
    case SOFTING_PB_PRO_DEVICE:
    case SOFTING_PB_PRO_104_DEVICE:
      return hw_pci_pro_probe(PciDev, PciId);
      break;
      
    case SOFTING_PB_PCIE_DEVICE:
      return hw_pcie_probe(PciDev, PciId);
      break;
      
    default:
      printk("profiboard: unknown hardware 0x%x line %u\n",PciId->subdevice, __LINE__);
      break;
       
  }
  
  
  return (-ENODEV);
}

/*****************************************************************************/
/* hw_pci_pro_probe                                                          */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   this function will be called when the system finds a new device         */
/*   on the bus which matches one of the device tables defined below         */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*    struct pci_dev       : contains the resource information for the       */
/*                           new device                                      */
/*    struct pci_device_id*: contains information about manufacturer,        */
/*                           device id ...                                   */
/*                                                                           */
/*****************************************************************************/

int hw_pci_pro_probe (struct pci_dev *PciDev, const struct pci_device_id *PciId)
    /*-------------------------------------------------------------------------*/
{
  int BoardIndex;
  char LocBuf[256];
  unsigned short BufSize=sizeof(LocBuf);
  int  Length;
  int                 PciResult;
  
  
  printk("hw_pci_pro_probe:\n");
  
  if ((PciId->subvendor != SOFTING_VENDOR_ID) || 
       ((PciId->subdevice != SOFTING_PB_PRO_DEVICE) && (PciId->subdevice != SOFTING_PB_PRO_104_DEVICE))
       
     )
  {                           // no. 
    printk("wrong PCI device: subvendor=0x%x, subdevice=0x%x\n",
           PciId->subvendor, PciId->subdevice);
    return (-ENODEV);
  }

  
  if(pci_enable_device(PciDev))
    return -1;

  
  for(BoardIndex=0; ((BoardIndex<PB_MAX_NR_BOARDS) && (CtrlExt[BoardIndex].isUsed)); BoardIndex++)
  {
    ;
  }
  

  
  if(BoardIndex >= PB_MAX_NR_BOARDS)
  {
    printk("too many boards\n");
    return -ENODEV;
  }
  
  memset(&CtrlExt[BoardIndex], 0, sizeof(CtrlExt[BoardIndex]));
  
  if(pci_resource_start(PciDev, 3))
  { // dual channel board
    BoardIndex++;
    if(BoardIndex >= PB_MAX_NR_BOARDS)
    {
      printk("too many boards\n");
      return -ENODEV;
    }
    
    if(CtrlExt[BoardIndex].isUsed)
    { // allready used!
      printk("too many boards\n");
      return -ENODEV;
    }
    CtrlExt [BoardIndex].Channel = 0;
  }

  
  printk("ProfiPCI board found: first DPRAM at: 0x%lx second DPRAM at: 0x%lx, IRQ: %u config space: 0x%lx\n",
         (PB_ADDRESS)pci_resource_start(PciDev, 2),
          (PB_ADDRESS)pci_resource_start(PciDev, 3),
         PciDev->irq,
         (PB_ADDRESS)pci_resource_start(PciDev, 1));
         
         
  CtrlExt [BoardIndex].DeviceType = PciId->subdevice;
  
  init_waitqueue_head (&CtrlExt [BoardIndex].Service.QRead);
  init_waitqueue_head (&CtrlExt [BoardIndex].Service.QWrite);
  init_waitqueue_head (&CtrlExt [BoardIndex].Data.QNewInputData);
  init_waitqueue_head (&CtrlExt [BoardIndex].SlaveIn.QInputAtController);
  
  init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_read, "wait_for_read");
  printk("wait_for_read = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_read);
  init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_write, "wait_for_write");
  printk("wait_for_write = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_write);
  init_my_wait(&CtrlExt [BoardIndex].Data.wait_for_new_input_data, "wait_for_new_input_data");
  printk("wait_for_new_input_data = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Data.wait_for_new_input_data);
  spin_lock_init(&CtrlExt [BoardIndex].Spinlock);
  
  
  CtrlExt [BoardIndex].interrupt = PciDev->irq;

    // map the DPRAM memory to kernel space 
   
  CtrlExt [BoardIndex].DprAddr = (PB_ADDRESS)ioremap_nocache(pci_resource_start(PciDev, 2), pci_resource_len(PciDev, 2));
  
  CtrlExt [BoardIndex].physicalDPRAM = pci_resource_start(PciDev, 2);
  
  if (!CtrlExt [BoardIndex].DprAddr)
  {
    printk("DPRAM mapping failed; ioremap returned NULL\n");
    return 0;
  }

  CtrlExt [BoardIndex].c_to_h_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + C_TO_H_IRQ_VAL_OFFSET_PRO);
  CtrlExt [BoardIndex].h_to_c_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + H_TO_C_IRQ_VAL_OFFSET_PRO);
  
  *CtrlExt [BoardIndex].c_to_h_irq_val = 0;
  *CtrlExt [BoardIndex].h_to_c_irq_val = 0;
  
  printk("DPRAM %u mapped to 0x%lx\n",BoardIndex, CtrlExt [BoardIndex].DprAddr);

    // map the configuration space  

  if (pci_resource_len(PciDev, 1) != REG_SIZE)
  {
    printk("configuration space mapping failed, length of configuration space: %lu bytes\n",
           (PB_ADDRESS)pci_resource_len(PciDev, 1));
    
    iounmap((VOID *) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].DprAddr = 0;
    return 0;
  }
  
  if(!request_region (pci_resource_start(PciDev, 1), pci_resource_len(PciDev, 1), PB_NAME))
  {
    printk("configuration space mapping failed; ioremap returned NULL\n");
    
    iounmap((VOID *) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].ConfigBase = 0;
    return 0;
    
  }
  
  CtrlExt [BoardIndex].ConfigBase = pci_resource_start(PciDev, 1);
  
  CtrlExt [BoardIndex].Channel = 0;
  
  CtrlExt [BoardIndex].isUsed  = TRUE;
  
  CtrlExt [BoardIndex].fwBufferPtr = 0;
  
  if ((PciResult = request_irq (CtrlExt [BoardIndex].interrupt,
       profibus_irq,
       IRQF_DISABLED | IRQF_SHARED,
       "PbproPCI",
       (void*) &CtrlExt [BoardIndex]))
     )
  {
    printk  ("can't get assigned irq %d error code %d\n",
             CtrlExt [BoardIndex].interrupt,
             PciResult);
    
    CtrlExt [BoardIndex].interrupt = 0;
  }
  CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
  CtrlExt [BoardIndex].asic = getPciOperationModeForAsic(&CtrlExt [BoardIndex], CtrlExt [BoardIndex].Channel);
  printk("ASIC TYPE %u\n",CtrlExt [BoardIndex].asic);
//  return 0;
  
  PbInitBoard( &CtrlExt [BoardIndex], 0 );
  
 // return 0;
  
  if( PbStartBoard( &CtrlExt [BoardIndex], C165_RESET ) >= 0 ) 
  {
    cmi_get_data(&CtrlExt [BoardIndex],ID_FW_VERS_IMAGE,0,&BufSize,LocBuf,
                  PB_KERNEL_SPACE );
    
    if(CtrlExt [BoardIndex].FWVersion)
    {
      kfree(CtrlExt [BoardIndex].FWVersion);
    }
    CtrlExt [BoardIndex].FWVersion = kmalloc( strlen(LocBuf)+1, GFP_ATOMIC );
    if( CtrlExt [BoardIndex].FWVersion ) 
    {
      memcpy( CtrlExt [BoardIndex].FWVersion, LocBuf, strlen(LocBuf) );
    } 
    else 
    {
      CtrlExt [BoardIndex].FWVersion[0]=0;
    }
    Length = sizeof(CtrlExt [BoardIndex].SerialNumber);
    cmi_get_data(&CtrlExt [BoardIndex],ID_SERIAL_DEVICE_NUMBER,0,(void *)&Length,
                  &CtrlExt [BoardIndex].SerialNumber, PB_KERNEL_SPACE );
  }
  
  

  
  PbInitBoard( &CtrlExt [BoardIndex], 0 );
  
 // return 0;
  
  tasklet_init(&CtrlExt [BoardIndex].PbSlaveTasklet, PbSlaveIST, (TASKPARATYPE) &CtrlExt [BoardIndex]);
  tasklet_init(&CtrlExt [BoardIndex].PbFirstTasklet, PbDataISTFirst, (TASKPARATYPE)&CtrlExt [BoardIndex]);
  tasklet_init(&CtrlExt [BoardIndex].PbDataTasklet, PbDataIST, (TASKPARATYPE)&CtrlExt [BoardIndex]);
  
  //memcpy(&HCtrlExt, &CtrlExt [BoardIndex], sizeof(struct CntrlExt));
  
  printk("Board %u initialized! IRQ=%u, config space = 0x%lx, DPRAM=0x%lx\n",   
         BoardIndex,
         CtrlExt [BoardIndex].interrupt,
         CtrlExt [BoardIndex].ConfigBase,
         CtrlExt [BoardIndex].DprAddr);
 
    
  if(pci_resource_start(PciDev, 3) /*&& (++BoardIndex < PB_MAX_NR_BOARDS)*/)
  {
    BoardIndex--;
    
    memset(&CtrlExt[BoardIndex], 0, sizeof(CtrlExt[BoardIndex]));
    
    CtrlExt [BoardIndex].interrupt  = PciDev->irq;
    CtrlExt [BoardIndex].Channel    = 1;
    
    init_waitqueue_head (&CtrlExt [BoardIndex].Service.QRead);
    init_waitqueue_head (&CtrlExt [BoardIndex].Service.QWrite);
    init_waitqueue_head (&CtrlExt [BoardIndex].Data.QNewInputData);
    init_waitqueue_head (&CtrlExt [BoardIndex].SlaveIn.QInputAtController);
    
    init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_read, "wait_for_read");
    printk("wait_for_read = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_read);
    init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_write, "wait_for_write");
    printk("wait_for_write = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_write);
    init_my_wait(&CtrlExt [BoardIndex].Data.wait_for_new_input_data, "wait_for_new_input_data");
    printk("wait_for_new_input_data = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Data.wait_for_new_input_data);
    spin_lock_init(&CtrlExt [BoardIndex].Spinlock);
    
 
    
    CtrlExt [BoardIndex].ConfigBase = pci_resource_start(PciDev, 1);
    
    CtrlExt [BoardIndex].DprAddr = (PB_ADDRESS) ioremap_nocache(pci_resource_start(PciDev, 3), pci_resource_len(PciDev, 3));
    CtrlExt [BoardIndex].DeviceType = PciId->subdevice;
    CtrlExt [BoardIndex].c_to_h_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + C_TO_H_IRQ_VAL_OFFSET_PRO);
    CtrlExt [BoardIndex].h_to_c_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + H_TO_C_IRQ_VAL_OFFSET_PRO);
    
    *CtrlExt [BoardIndex].c_to_h_irq_val = 0;
    *CtrlExt [BoardIndex].h_to_c_irq_val = 0;

    printk("DPRAM %u mapped to 0x%lx\n",BoardIndex, CtrlExt [BoardIndex].DprAddr);

  
    if (!CtrlExt [BoardIndex].DprAddr)
    {
      printk("DPRAM mapping of channel 2 failed; ioremap returned NULL\n");
    }
    
    CtrlExt [BoardIndex].isUsed = TRUE;
    CtrlExt [BoardIndex].fwBufferPtr = 0;
    
    CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
    CtrlExt [BoardIndex].asic = getPciOperationModeForAsic(&CtrlExt [BoardIndex], CtrlExt [BoardIndex].Channel);
    printk("ASIC TYPE %u\n",CtrlExt [BoardIndex].asic);
    
    if ((PciResult = request_irq (CtrlExt [BoardIndex].interrupt,
         profibus_irq,
         IRQF_DISABLED | IRQF_SHARED,
         "PbproPCI",
         (void*) &CtrlExt [BoardIndex]))
       )
    {
      printk  ("can't get assigned irq %d error code %d\n",
               CtrlExt [BoardIndex].interrupt,
               PciResult);
      
      CtrlExt [BoardIndex].interrupt = 0;
    }
    
    CtrlExt [BoardIndex].physicalDPRAM = pci_resource_start(PciDev, 3);
    
    PbInitBoard( &CtrlExt [BoardIndex], 0 );

    if( PbStartBoard( &CtrlExt [BoardIndex], C165_RESET ) >= 0 ) 
    {
      cmi_get_data(&CtrlExt [BoardIndex],ID_FW_VERS_IMAGE,0,&BufSize,LocBuf,
                    PB_KERNEL_SPACE );
      if(CtrlExt [BoardIndex].FWVersion)
      {
        kfree(CtrlExt [BoardIndex].FWVersion);
      }
      
      CtrlExt [BoardIndex].FWVersion = kmalloc( strlen(LocBuf)+1, GFP_ATOMIC );
      if( CtrlExt [BoardIndex].FWVersion ) 
      {
        memcpy( CtrlExt [BoardIndex].FWVersion, LocBuf, strlen(LocBuf) );
      } 
      else 
      {
        //NULL-POINTER!!!
        // CtrlExt [BoardIndex].FWVersion[0]=0;
      }
      Length = sizeof(CtrlExt [BoardIndex].SerialNumber);
      cmi_get_data(&CtrlExt [BoardIndex],ID_SERIAL_DEVICE_NUMBER,0,(void *)&Length,
                    &CtrlExt [BoardIndex].SerialNumber, PB_KERNEL_SPACE );
    }
    PbInitBoard( &CtrlExt [BoardIndex], 0 ); 
    
    
    
    tasklet_init(&CtrlExt [BoardIndex].PbSlaveTasklet, PbSlaveIST, (TASKPARATYPE) &CtrlExt [BoardIndex]);
    tasklet_init(&CtrlExt [BoardIndex].PbFirstTasklet, PbDataISTFirst, (TASKPARATYPE)&CtrlExt [BoardIndex]);
    tasklet_init(&CtrlExt [BoardIndex].PbDataTasklet, PbDataIST, (TASKPARATYPE)&CtrlExt [BoardIndex]);

    //memcpy(&CtrlExt [BoardIndex-1], &CtrlExt [BoardIndex], sizeof(struct CntrlExt));
    //memcpy(&CtrlExt [BoardIndex], &HCtrlExt, sizeof(struct CntrlExt));

    printk("Board %u initialized! IRQ=%u, config space = 0x%lx, DPRAM=0x%lx\n",  
           BoardIndex, 
           CtrlExt [BoardIndex].interrupt,
           CtrlExt [BoardIndex].ConfigBase,
           CtrlExt [BoardIndex].DprAddr);
    
  }
  
  
  
  
  return 0;
 
}



/*****************************************************************************/
/* hw_pci_probe                                                              */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   this function will be called when the system finds a new device         */
/*   on the bus which matches one of the device tables defined below         */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*    struct pci_dev       : contains the resource information for the       */
/*                           new device                                      */
/*    struct pci_device_id*: contains information about manufacturer,        */
/*                           device id ...                                   */
/*                                                                           */
/*****************************************************************************/

int hw_pci_probe (struct pci_dev *PciDev, const struct pci_device_id *PciId)
    /*-------------------------------------------------------------------------*/
{
  int BoardIndex;
  char LocBuf[256];
  unsigned short BufSize=sizeof(LocBuf);
  int  Length;
  int                 PciResult;
  
  
  
  printk("hw_pci_probe:\n");
  
  if(pci_enable_device(PciDev))
    return -1;

  
  for(BoardIndex=0; ((BoardIndex<PB_MAX_NR_BOARDS) && (CtrlExt[BoardIndex].isUsed)); BoardIndex++)
  {
    ;
  }
  
  if(BoardIndex >= PB_MAX_NR_BOARDS)
  {
    printk("too many boards\n");
    return -ENODEV;
  }
  
  memset(&CtrlExt[BoardIndex], 0, sizeof(CtrlExt[BoardIndex]));
  
  if (PciId->subvendor != SOFTING_VENDOR_ID)
  {                           // no. 
    printk("wrong PCI device: subvendor=0x%x, subdevice=0x%x\n",
           PciId->subvendor, PciId->subdevice);
    return (-ENODEV);
  }
  
  printk("ProfiPCI board found: first DPRAM at: 0x%lx second DPRAM at: 0x%lx, IRQ: %u config space: 0x%lx\n",
         (PB_ADDRESS)pci_resource_start(PciDev, 2),
          (PB_ADDRESS)pci_resource_start(PciDev, 3),
         PciDev->irq,
         (PB_ADDRESS)pci_resource_start(PciDev, 1));
         
         
  CtrlExt [BoardIndex].DeviceType = PciId->subdevice;
  
  init_waitqueue_head (&CtrlExt [BoardIndex].Service.QRead);
  init_waitqueue_head (&CtrlExt [BoardIndex].Service.QWrite);
  init_waitqueue_head (&CtrlExt [BoardIndex].Data.QNewInputData);
  init_waitqueue_head (&CtrlExt [BoardIndex].SlaveIn.QInputAtController);
  
  init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_read, "wait_for_read");
  printk("wait_for_read = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_read);
  init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_write, "wait_for_write");
  printk("wait_for_write = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_write);
  init_my_wait(&CtrlExt [BoardIndex].Data.wait_for_new_input_data, "wait_for_new_input_data");
  printk("wait_for_new_input_data = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Data.wait_for_new_input_data);
  spin_lock_init(&CtrlExt [BoardIndex].Spinlock);
  
  
  CtrlExt [BoardIndex].interrupt = PciDev->irq;

    // map the DPRAM memory to kernel space 
   
  CtrlExt [BoardIndex].DprAddr = (PB_ADDRESS) ioremap_nocache(pci_resource_start(PciDev, 2), pci_resource_len(PciDev, 2));
  
  CtrlExt [BoardIndex].physicalDPRAM = pci_resource_start(PciDev, 2);
  
  if (!CtrlExt [BoardIndex].DprAddr)
  {
    printk("DPRAM mapping failed; ioremap returned NULL\n");
    return 0;
  }

  CtrlExt [BoardIndex].c_to_h_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + C_TO_H_IRQ_VAL_OFFSET);
  CtrlExt [BoardIndex].h_to_c_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + H_TO_C_IRQ_VAL_OFFSET);
  
  *CtrlExt [BoardIndex].c_to_h_irq_val = 0;
  *CtrlExt [BoardIndex].h_to_c_irq_val = 0;
  
  printk("DPRAM %u mapped to 0x%lx\n",BoardIndex, CtrlExt [BoardIndex].DprAddr);

    // map the configuration space  

  if (pci_resource_len(PciDev, 1) != REG_SIZE)
  {
    printk("configuration space mapping failed, length of configuration space: %lu bytes\n",
           (PB_ADDRESS)pci_resource_len(PciDev, 1));
    
    iounmap((VOID *) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].DprAddr = 0;
    return 0;
  }
  
  if(!request_region (pci_resource_start(PciDev, 1), pci_resource_len(PciDev, 1), PB_NAME))
  {
    printk("configuration space mapping failed; ioremap returned NULL\n");
    
    iounmap((VOID *) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].ConfigBase = 0;
    return 0;
    
  }
  
  
  CtrlExt [BoardIndex].ConfigBase = pci_resource_start(PciDev, 1);
  
  CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
  
  CtrlExt [BoardIndex].Channel = 0;
  
  CtrlExt [BoardIndex].isUsed  = TRUE;
  
  CtrlExt [BoardIndex].fwBufferPtr = 0;
  
  CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
  CtrlExt [BoardIndex].asic = getPciOperationModeForAsic(&CtrlExt [BoardIndex], CtrlExt [BoardIndex].Channel);
  printk("ASIC TYPE %u\n",CtrlExt [BoardIndex].asic);
  
  if ((PciResult = request_irq (CtrlExt [BoardIndex].interrupt,
       profibus_irq,
       IRQF_DISABLED | IRQF_SHARED,
       "PbPCI",
       (void*) &CtrlExt [BoardIndex]))
     )
  {
    printk  ("can't get assigned irq %d error code %d\n",
             CtrlExt [BoardIndex].interrupt,
             PciResult);
    
    CtrlExt [BoardIndex].interrupt = 0;
  }
  
  //return 0;
  
  PbInitBoard( &CtrlExt [BoardIndex], 0 );
  
 // return 0;
  
  if( PbStartBoard( &CtrlExt [BoardIndex], C165_RESET ) >= 0 ) 
  {
    cmi_get_data(&CtrlExt [BoardIndex],ID_FW_VERS_IMAGE,0,&BufSize,LocBuf,
                  PB_KERNEL_SPACE );
    
    if(CtrlExt [BoardIndex].FWVersion)
    {
      kfree(CtrlExt [BoardIndex].FWVersion);
    }
    CtrlExt [BoardIndex].FWVersion = kmalloc( strlen(LocBuf)+1, GFP_ATOMIC );
   
    if( CtrlExt [BoardIndex].FWVersion ) 
    {
      memcpy( CtrlExt [BoardIndex].FWVersion, LocBuf, strlen(LocBuf) );
    } 
    else 
    {
      CtrlExt [BoardIndex].FWVersion[0]=0;
    }
    Length = sizeof(CtrlExt [BoardIndex].SerialNumber);
    cmi_get_data(&CtrlExt [BoardIndex],ID_SERIAL_DEVICE_NUMBER,0,(void *)&Length,
                  &CtrlExt [BoardIndex].SerialNumber, PB_KERNEL_SPACE );
  }
  
  

  
  PbInitBoard( &CtrlExt [BoardIndex], 0 );
  
 // return 0;
  
  tasklet_init(&CtrlExt [BoardIndex].PbSlaveTasklet, PbSlaveIST, (TASKPARATYPE) &CtrlExt [BoardIndex]);
  tasklet_init(&CtrlExt [BoardIndex].PbFirstTasklet, PbDataISTFirst, (TASKPARATYPE)&CtrlExt [BoardIndex]);
  tasklet_init(&CtrlExt [BoardIndex].PbDataTasklet, PbDataIST, (TASKPARATYPE)&CtrlExt [BoardIndex]);
  
  //DumpMem((void*)CtrlExt [BoardIndex].DprAddr,0x100,"DPRAM");
  printk("Board %u initialized! IRQ=%u, config space = 0x%lx, DPRAM=0x%lx\n",   
         BoardIndex,
         CtrlExt [BoardIndex].interrupt,
         CtrlExt [BoardIndex].ConfigBase,
         CtrlExt [BoardIndex].DprAddr);
  
  //memcpy(&HCtrlExt, &CtrlExt [BoardIndex], sizeof(HCtrlExt));
 
    
  if(pci_resource_start(PciDev, 3) && (++BoardIndex < PB_MAX_NR_BOARDS))
  {
    memset(&CtrlExt[BoardIndex], 0, sizeof(CtrlExt[BoardIndex]));
    
    CtrlExt [BoardIndex].interrupt  = PciDev->irq;
    CtrlExt [BoardIndex].Channel    = 1;
    
    init_waitqueue_head (&CtrlExt [BoardIndex].Service.QRead);
    init_waitqueue_head (&CtrlExt [BoardIndex].Service.QWrite);
    init_waitqueue_head (&CtrlExt [BoardIndex].Data.QNewInputData);
    init_waitqueue_head (&CtrlExt [BoardIndex].SlaveIn.QInputAtController);
    
    init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_read, "wait_for_read");
    printk("wait_for_read = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_read);
    init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_write, "wait_for_write");
    printk("wait_for_write = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_write);
    init_my_wait(&CtrlExt [BoardIndex].Data.wait_for_new_input_data, "wait_for_new_input_data");
    printk("wait_for_new_input_data = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Data.wait_for_new_input_data);
    spin_lock_init(&CtrlExt [BoardIndex].Spinlock);
    
 
    
    CtrlExt [BoardIndex].ConfigBase = CtrlExt [BoardIndex-1].ConfigBase;
    
    CtrlExt [BoardIndex].DprAddr = (PB_ADDRESS) ioremap_nocache(pci_resource_start(PciDev, 3), pci_resource_len(PciDev, 3));
    
    CtrlExt [BoardIndex].DeviceType = PciId->subdevice;
    CtrlExt [BoardIndex].c_to_h_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + C_TO_H_IRQ_VAL_OFFSET);
    CtrlExt [BoardIndex].h_to_c_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + H_TO_C_IRQ_VAL_OFFSET);
    
    *CtrlExt [BoardIndex].c_to_h_irq_val = 0;
    *CtrlExt [BoardIndex].h_to_c_irq_val = 0;
  
    printk("DPRAM %u mapped to 0x%lx\n",BoardIndex, CtrlExt [BoardIndex].DprAddr);

  
    if (!CtrlExt [BoardIndex].DprAddr)
    {
      printk("DPRAM mapping of channel 2 failed; ioremap returned NULL\n");
    }
    
    CtrlExt [BoardIndex].isUsed = TRUE;
    CtrlExt [BoardIndex].fwBufferPtr = 0;
    
    CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
    
    CtrlExt [BoardIndex].asic = getPciOperationModeForAsic(&CtrlExt [BoardIndex], CtrlExt [BoardIndex].Channel);
    printk("ASIC TYPE %u\n",CtrlExt [BoardIndex].asic);
    
    if ((PciResult = request_irq (CtrlExt [BoardIndex].interrupt,
         profibus_irq,
         IRQF_DISABLED | IRQF_SHARED,
         "PbPCI",
         (void*) &CtrlExt [BoardIndex]))
       )
    {
      printk  ("can't get assigned irq %d error code %d\n",
               CtrlExt [BoardIndex].interrupt,
               PciResult);
      
      CtrlExt [BoardIndex].interrupt = 0;
    }
    
    CtrlExt [BoardIndex].physicalDPRAM = pci_resource_start(PciDev, 3);
    
    PbInitBoard( &CtrlExt [BoardIndex], 0 );

    if( PbStartBoard( &CtrlExt [BoardIndex], C165_RESET ) >= 0 ) 
    {
      cmi_get_data(&CtrlExt [BoardIndex],ID_FW_VERS_IMAGE,0,&BufSize,LocBuf,
                    PB_KERNEL_SPACE );
      if(CtrlExt [BoardIndex].FWVersion)
      {
        kfree(CtrlExt [BoardIndex].FWVersion);
      }
      
      CtrlExt [BoardIndex].FWVersion = kmalloc( strlen(LocBuf)+1, GFP_ATOMIC );
      if( CtrlExt [BoardIndex].FWVersion ) 
      {
        memcpy( CtrlExt [BoardIndex].FWVersion, LocBuf, strlen(LocBuf) );
      } 
      else 
      {
        //NULL-POINTER!!!
        // CtrlExt [BoardIndex].FWVersion[0]=0;
      }
      Length = sizeof(CtrlExt [BoardIndex].SerialNumber);
      cmi_get_data(&CtrlExt [BoardIndex],ID_SERIAL_DEVICE_NUMBER,0,(void *)&Length,
                    &CtrlExt [BoardIndex].SerialNumber, PB_KERNEL_SPACE );
    }
    PbInitBoard( &CtrlExt [BoardIndex], 0 ); 
    
    
    
    tasklet_init(&CtrlExt [BoardIndex].PbSlaveTasklet, PbSlaveIST, (TASKPARATYPE) &CtrlExt [BoardIndex]);
    tasklet_init(&CtrlExt [BoardIndex].PbFirstTasklet, PbDataISTFirst, (TASKPARATYPE)&CtrlExt [BoardIndex]);
    tasklet_init(&CtrlExt [BoardIndex].PbDataTasklet, PbDataIST, (TASKPARATYPE)&CtrlExt [BoardIndex]);

   // DumpMem((void*)CtrlExt [BoardIndex].DprAddr,0x100,"DPRAM");
    
    // exchange of channels because of the physical layout
  //  memcpy(&CtrlExt [BoardIndex-1], &CtrlExt [BoardIndex], sizeof(HCtrlExt));
   // memcpy(&CtrlExt [BoardIndex], &HCtrlExt, sizeof(HCtrlExt));
   
    printk("Board %u initialized! IRQ=%u, config space = 0x%lx, DPRAM=0x%lx\n",  
           BoardIndex, 
           CtrlExt [BoardIndex].interrupt,
           CtrlExt [BoardIndex].ConfigBase,
           CtrlExt [BoardIndex].DprAddr);
    
  }
  
  
  
  
  return 0;
 
}

/*****************************************************************************/
/* hw_pcie_probe                                                             */
/*****************************************************************************/
/* FUNCTIONAL DESCRIPTION                                                    */
/*   this function will be called when the system finds a new device         */
/*   on the bus which matches one of the device tables defined below         */
/*****************************************************************************/
/* PARAMETER DECRIPTION                                                      */
/*    struct pci_dev       : contains the resource information for the       */
/*                           new device                                      */
/*    struct pci_device_id*: contains information about manufacturer,        */
/*                           device id ...                                   */
/*                                                                           */
/*****************************************************************************/

int hw_pcie_probe (struct pci_dev *PciDev, const struct pci_device_id *PciId)
    /*-------------------------------------------------------------------------*/
{
  int BoardIndex;
  char LocBuf[256];
  unsigned short BufSize=sizeof(LocBuf);
  int  Length;
  int                 PciResult;
  
  
  printk("hw_pcie_probe:\n");
  
  if (PciId->subvendor != SOFTING_VENDOR_ID)
  {                           // no. 
    printk("wrong PCI device: subvendor=0x%x, subdevice=0x%x\n",
           PciId->subvendor, PciId->subdevice);
    return (-ENODEV);
  }

  
  if(pci_enable_device(PciDev))
    return -1;

  
  for(BoardIndex=0; ((BoardIndex<PB_MAX_NR_BOARDS) && (CtrlExt[BoardIndex].isUsed)); BoardIndex++)
  {
    ;
  }
  

  
  if(BoardIndex >= PB_MAX_NR_BOARDS)
  {
    printk("too many boards\n");
    return -ENODEV;
  }
  
  memset(&CtrlExt[BoardIndex], 0, sizeof(CtrlExt[BoardIndex]));
  
  if(pci_resource_start(PciDev, 3))
  { // dual channel board
    BoardIndex++;
    if(BoardIndex >= PB_MAX_NR_BOARDS)
    {
      printk("too many boards\n");
      return -ENODEV;
    }
    
    if(CtrlExt[BoardIndex].isUsed)
    { // allready used!
      printk("too many boards\n");
      return -ENODEV;
    }
    CtrlExt [BoardIndex].Channel = 0;
  }

  
  printk("ProfiPCIe board found: first DPRAM at: 0x%lx second DPRAM at: 0x%lx, IRQ: %u config space: 0x%lx\n",
         (PB_ADDRESS)pci_resource_start(PciDev, 2),
          (PB_ADDRESS)pci_resource_start(PciDev, 3),
         PciDev->irq,
         (PB_ADDRESS)pci_resource_start(PciDev, 1));
         
         
  CtrlExt [BoardIndex].DeviceType = PciId->subdevice;
  
  init_waitqueue_head (&CtrlExt [BoardIndex].Service.QRead);
  init_waitqueue_head (&CtrlExt [BoardIndex].Service.QWrite);
  init_waitqueue_head (&CtrlExt [BoardIndex].Data.QNewInputData);
  init_waitqueue_head (&CtrlExt [BoardIndex].SlaveIn.QInputAtController);
  
  init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_read, "wait_for_read");
  printk("wait_for_read = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_read);
  init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_write, "wait_for_write");
  printk("wait_for_write = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_write);
  init_my_wait(&CtrlExt [BoardIndex].Data.wait_for_new_input_data, "wait_for_new_input_data");
  printk("wait_for_new_input_data = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Data.wait_for_new_input_data);
  spin_lock_init(&CtrlExt [BoardIndex].Spinlock);
  
  
  CtrlExt [BoardIndex].interrupt = PciDev->irq;

    // map the DPRAM memory to kernel space 
   
  CtrlExt [BoardIndex].DprAddr = (PB_ADDRESS)ioremap_nocache(pci_resource_start(PciDev, 2), pci_resource_len(PciDev, 2));
  
  CtrlExt [BoardIndex].physicalDPRAM = pci_resource_start(PciDev, 2);
  
  if (!CtrlExt [BoardIndex].DprAddr)
  {
    printk("DPRAM mapping failed; ioremap returned NULL\n");
    return 0;
  }

  CtrlExt [BoardIndex].c_to_h_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + C_TO_H_IRQ_VAL_OFFSET_PCIE);
  CtrlExt [BoardIndex].h_to_c_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + H_TO_C_IRQ_VAL_OFFSET_PCIE);
  
  *CtrlExt [BoardIndex].c_to_h_irq_val = 0;
  *CtrlExt [BoardIndex].h_to_c_irq_val = 0;
  
  printk("DPRAM %u mapped to 0x%lx\n",BoardIndex, CtrlExt [BoardIndex].DprAddr);

    // map the configuration space  

  if (pci_resource_len(PciDev, 1) != REG_SIZE_PCIE)
  {
    printk("configuration space mapping failed, length of configuration space: %lu bytes\n",
           (PB_ADDRESS)pci_resource_len(PciDev, 1));
    
    iounmap((VOID *) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].DprAddr = 0;
    return 0;
  }
  
  if(!request_region (pci_resource_start(PciDev, 1), pci_resource_len(PciDev, 1), PB_NAME))
  {
    printk("configuration space mapping failed; ioremap returned NULL\n");
    
    iounmap((VOID *) CtrlExt [BoardIndex].DprAddr);
    CtrlExt [BoardIndex].ConfigBase = 0;
    return 0;
    
  }
  
  CtrlExt [BoardIndex].ConfigBase = pci_resource_start(PciDev, 1);
  
  CtrlExt [BoardIndex].Channel = 0;
  
  CtrlExt [BoardIndex].isUsed  = TRUE;
  
  CtrlExt [BoardIndex].fwBufferPtr = 0;
  
  if ((PciResult = request_irq (CtrlExt [BoardIndex].interrupt,
       profibus_irq,
       IRQF_DISABLED | IRQF_SHARED,
       "PbproPCIe",
       (void*) &CtrlExt [BoardIndex]))
     )
  {
    printk  ("can't get assigned irq %d error code %d\n",
             CtrlExt [BoardIndex].interrupt,
             PciResult);
    
    CtrlExt [BoardIndex].interrupt = 0;
  }
  CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
  CtrlExt [BoardIndex].asic = getPciOperationModeForAsic(&CtrlExt [BoardIndex], CtrlExt [BoardIndex].Channel);
  printk("ASIC TYPE %u\n",CtrlExt [BoardIndex].asic);
  
  PbInitBoard( &CtrlExt [BoardIndex], 0 );
  
  
  if( PbStartBoard( &CtrlExt [BoardIndex], C165_RESET ) >= 0 ) 
  {
    cmi_get_data(&CtrlExt [BoardIndex],ID_FW_VERS_IMAGE,0,&BufSize,LocBuf,
                  PB_KERNEL_SPACE );
    
    if(CtrlExt [BoardIndex].FWVersion)
    {
      kfree(CtrlExt [BoardIndex].FWVersion);
    }
    CtrlExt [BoardIndex].FWVersion = kmalloc( strlen(LocBuf)+1, GFP_ATOMIC );
    if( CtrlExt [BoardIndex].FWVersion ) 
    {
      memcpy( CtrlExt [BoardIndex].FWVersion, LocBuf, strlen(LocBuf) );
    } 
    else 
    {
      CtrlExt [BoardIndex].FWVersion[0]=0;
    }
    Length = sizeof(CtrlExt [BoardIndex].SerialNumber);
    cmi_get_data(&CtrlExt [BoardIndex],ID_SERIAL_DEVICE_NUMBER,0,(void *)&Length,
                  &CtrlExt [BoardIndex].SerialNumber, PB_KERNEL_SPACE );
  }
  
  

  
  PbInitBoard( &CtrlExt [BoardIndex], 0 );
  
 // return 0;
  
  tasklet_init(&CtrlExt [BoardIndex].PbSlaveTasklet, PbSlaveIST, (TASKPARATYPE) &CtrlExt [BoardIndex]);
  tasklet_init(&CtrlExt [BoardIndex].PbFirstTasklet, PbDataISTFirst, (TASKPARATYPE)&CtrlExt [BoardIndex]);
  tasklet_init(&CtrlExt [BoardIndex].PbDataTasklet, PbDataIST, (TASKPARATYPE)&CtrlExt [BoardIndex]);
  
  //memcpy(&HCtrlExt, &CtrlExt [BoardIndex], sizeof(struct CntrlExt));
  
  printk("Board %u initialized! IRQ=%u, config space = 0x%lx, DPRAM=0x%lx\n",   
         BoardIndex,
         CtrlExt [BoardIndex].interrupt,
         CtrlExt [BoardIndex].ConfigBase,
         CtrlExt [BoardIndex].DprAddr);
 
    
  if(pci_resource_start(PciDev, 3) /*&& (++BoardIndex < PB_MAX_NR_BOARDS)*/)
  {
    BoardIndex--;
    
    memset(&CtrlExt[BoardIndex], 0, sizeof(CtrlExt[BoardIndex]));
    
    CtrlExt [BoardIndex].interrupt  = PciDev->irq;
    CtrlExt [BoardIndex].Channel    = 1;
    
    init_waitqueue_head (&CtrlExt [BoardIndex].Service.QRead);
    init_waitqueue_head (&CtrlExt [BoardIndex].Service.QWrite);
    init_waitqueue_head (&CtrlExt [BoardIndex].Data.QNewInputData);
    init_waitqueue_head (&CtrlExt [BoardIndex].SlaveIn.QInputAtController);
    
    init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_read, "wait_for_read");
    printk("wait_for_read = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_read);
    init_my_wait(&CtrlExt [BoardIndex].Service.wait_for_write, "wait_for_write");
    printk("wait_for_write = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Service.wait_for_write);
    init_my_wait(&CtrlExt [BoardIndex].Data.wait_for_new_input_data, "wait_for_new_input_data");
    printk("wait_for_new_input_data = 0x%lx\n",(unsigned long)&CtrlExt [BoardIndex].Data.wait_for_new_input_data);
    spin_lock_init(&CtrlExt [BoardIndex].Spinlock);
    
 
    
    CtrlExt [BoardIndex].ConfigBase = pci_resource_start(PciDev, 1);
    
    CtrlExt [BoardIndex].DprAddr = (PB_ADDRESS) ioremap_nocache(pci_resource_start(PciDev, 3), pci_resource_len(PciDev, 3));
    CtrlExt [BoardIndex].DeviceType = PciId->subdevice;
    CtrlExt [BoardIndex].c_to_h_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + C_TO_H_IRQ_VAL_OFFSET_PCIE);
    CtrlExt [BoardIndex].h_to_c_irq_val =  (unsigned short *)(CtrlExt [BoardIndex].DprAddr + H_TO_C_IRQ_VAL_OFFSET_PCIE);
    
    *CtrlExt [BoardIndex].c_to_h_irq_val = 0;
    *CtrlExt [BoardIndex].h_to_c_irq_val = 0;

    printk("DPRAM %u mapped to 0x%lx\n",BoardIndex, CtrlExt [BoardIndex].DprAddr);

  
    if (!CtrlExt [BoardIndex].DprAddr)
    {
      printk("DPRAM mapping of channel 2 failed; ioremap returned NULL\n");
    }
    
    CtrlExt [BoardIndex].isUsed = TRUE;
    CtrlExt [BoardIndex].fwBufferPtr = 0;
    
    CtrlExt [BoardIndex].config_space_mem = pci_resource_start(PciDev, 0);
    CtrlExt [BoardIndex].asic = getPciOperationModeForAsic(&CtrlExt [BoardIndex], CtrlExt [BoardIndex].Channel);
    printk("ASIC TYPE %u\n",CtrlExt [BoardIndex].asic);
    
    if ((PciResult = request_irq (CtrlExt [BoardIndex].interrupt,
         profibus_irq,
         IRQF_DISABLED | IRQF_SHARED,
         "PbproPCIe",
         (void*) &CtrlExt [BoardIndex]))
       )
    {
      printk  ("can't get assigned irq %d error code %d\n",
               CtrlExt [BoardIndex].interrupt,
               PciResult);
      
      CtrlExt [BoardIndex].interrupt = 0;
    }
    
    CtrlExt [BoardIndex].physicalDPRAM = pci_resource_start(PciDev, 3);
    
    PbInitBoard( &CtrlExt [BoardIndex], 0 );

    if( PbStartBoard( &CtrlExt [BoardIndex], C165_RESET ) >= 0 ) 
    {
      cmi_get_data(&CtrlExt [BoardIndex],ID_FW_VERS_IMAGE,0,&BufSize,LocBuf,
                    PB_KERNEL_SPACE );
      if(CtrlExt [BoardIndex].FWVersion)
      {
        kfree(CtrlExt [BoardIndex].FWVersion);
      }
      
      CtrlExt [BoardIndex].FWVersion = kmalloc( strlen(LocBuf)+1, GFP_ATOMIC );
      if( CtrlExt [BoardIndex].FWVersion ) 
      {
        memcpy( CtrlExt [BoardIndex].FWVersion, LocBuf, strlen(LocBuf) );
      } 
      else 
      {
        //NULL-POINTER!!!
        // CtrlExt [BoardIndex].FWVersion[0]=0;
      }
      Length = sizeof(CtrlExt [BoardIndex].SerialNumber);
      cmi_get_data(&CtrlExt [BoardIndex],ID_SERIAL_DEVICE_NUMBER,0,(void *)&Length,
                    &CtrlExt [BoardIndex].SerialNumber, PB_KERNEL_SPACE );
    }
    PbInitBoard( &CtrlExt [BoardIndex], 0 ); 
    
    
    
    tasklet_init(&CtrlExt [BoardIndex].PbSlaveTasklet, PbSlaveIST, (TASKPARATYPE) &CtrlExt [BoardIndex]);
    tasklet_init(&CtrlExt [BoardIndex].PbFirstTasklet, PbDataISTFirst, (TASKPARATYPE)&CtrlExt [BoardIndex]);
    tasklet_init(&CtrlExt [BoardIndex].PbDataTasklet, PbDataIST, (TASKPARATYPE)&CtrlExt [BoardIndex]);

    //memcpy(&CtrlExt [BoardIndex-1], &CtrlExt [BoardIndex], sizeof(struct CntrlExt));
    //memcpy(&CtrlExt [BoardIndex], &HCtrlExt, sizeof(struct CntrlExt));

    printk("Board %u initialized! IRQ=%u, config space = 0x%lx, DPRAM=0x%lx\n",  
           BoardIndex, 
           CtrlExt [BoardIndex].interrupt,
           CtrlExt [BoardIndex].ConfigBase,
           CtrlExt [BoardIndex].DprAddr);
    
  }
  
  
  
  
  return 0;
 
}

int init_module()
{
  
  int ret=0;
  
  printk( Version );
  printk( Copyright );
  printk( Warranty );
  
  // initialize structures (esp. WaitQueue)
  
  sema_init(&PbGlobalSema, 1);        // only 1 task may hold the semaphore
  
  memset( CtrlExt, 0, sizeof(CtrlExt) );

  ret = initProfiPciDriver();
  
  spin_lock_init(&PlxSpinlock);
    
  return ret; 
}

void cleanup_module()
{
  pci_unregister_driver(&HwPci);

 
  unregister_chrdev(major_number, PB_NAME);
  
}

#endif // PB_PCI_SUPPORT 

