/*****************************************************************************/
/*                                                                           */
/*                               SOFTING AG                                  */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*                    Copyright (C) SOFTING AG 1997-2003                     */
/*                            All Rights Reserved                            */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Filename    : DPDEMO.H                                                   */
/*  Version     : 5.24                                                       */
/*  Date        : 31.07.2001                                                 */
/*  Author      : SOFTING AG                                                 */
/*                                                                           */
/*  Description : DP demo application configuration parameters               */
/*                                                                           */
/*  CHANGE_NOTES                                                             */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/


#ifndef __DPDEMO__
#define __DPDEMO__

/****************************************************************************/
/*** DP MASTER BUS PARAMETER SET ********************************************/
/****************************************************************************/

#define DP_MASTER_ADDRESS         0
#define PROFIBOARD_DP_MASTER_IDENT_NUMBER    0xB203
#define PROFICARD_DP_MASTER_IDENT_NUMBER     0xB204

#define DP_MASTER_IDENT_NUMBER               PROFIBOARD_DP_MASTER_IDENT_NUMBER


/*--------------------------------------------------------------------------*/

static const OCTET    FAR dp_bus_para_set [66] =
{
  0, 66,                                                    /* bus_para_len */
  1,                                                            /* fdl_add */
  DP_KBAUD_93_75,                                                 /* baud_rate */
  0x0C, 0x2C,                                                  /* Tsl = 300 */
  0x00, 0x20,                                              /* Min Tsdr = 15 */
  0x03, 0xff,                                             /* Max Tsdr = 150 */
  0,                                                                /* Tqui */
  150,                                                                /* Tset */
  0x00, 0x00, 0xC3, 0x50,                                            /* Ttr */
  10,                                                                  /* G */
  1,                                                                 /* HSA */
  4,                                                     /* max_retry_limit */
  0x00,/* DP_BP_ERROR_ACTION, */                      /* bp_flag: Autoclear */
  0, 1,                                   /* min_slave_interval [100micros] */
  0x03, 0xE8,                                        /* poll_timeout = 1000 */
  0x00, 0x64,                                    /* data_control_time = 100 */
  0, 0, 0, 0, 0, 0,                                         /* reserved [6] */
  0x00, DP_MASTER_USER_DATA_LEN,                    /* master_user_data_len */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

}; /* dp_bus_para_set */

/****************************************************************************/
/*** DP SLAVE EXAMPLE PARAMETER SETS ****************************************/
/****************************************************************************/
/*                                                                          */
/* slave_para_len: 16 +                                                     */
/*                 prm_data_len +                                           */
/*                 cfg_data_len +                                           */
/*                 aat_data_len +                                           */
/*                 slave_user_data_len +                                    */
/*                 number_alignment_bytes                                   */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*For every slave type an array has to be filled containing all             */
/*configuration and parametrization information.                            */
/*                                                                          */
/*To append a slave to the list, it is recommended to copy one array,       */
/*and to modify the contents according to the data required.                */
/*                                                                          */
/*First the ident_nr will have to be set to its new value. After that, enter*/
/*user_prm_data your slave requires. Next, enter the new configuration      */
/*data in the appropriate section. All other elements may remain unchanged  */
/*for a first trial.                                                        */
/*Finally, you should check all length information (of the array in total   */
/*as well as of its substructures). Bear in mind that length information    */
/*also has to be counted.                                                   */
/*Note: If an empty array was defined, the length would be 0x00, 0x02 as    */
/*      length information consumes 2 Bytes.                                */
/*                                                                          */
/*After the slave has been defined, an address has to be assigned.          */
/*This is done in strcuture slave_para_table by replacing the NULL pointer  */
/*by a pointer to the appropriate slave_structure.                          */
/*                                                                          */
/*The last step is to determine the slaves which are to be polled actively. */
/*This is accomplished by entering the slave_address in to array            */
/*dp_slave_table. The last entry of this table must be 0xFF.                */
/*                                                                          */
/*Notes: 1. To include the changes in the .exe you have to recompile        */
/*          the application.                                                */
/*       2. Error codes being displayed may be looked up in the DP section  */
/*          manual                                                          */
/****************************************************************************/
/****************************************************************************/

/****************************************************************************/
/*--------------------------------------------------------------------------*/
/*--- SIEMENS AG95U --------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_siemens_800C [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 39,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM /* | DP_SL_ACTIVE */,                            /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 9,                                            /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x80, 0x0C,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 4,                                            /* USIGN16: cfg_data_len */
  0x5F,                                                         /* cfg_data */
  0x6F,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  32,                                                      /* number_inputs */
  32,                                                     /* number_outputs */
  0, 2,                                           /* USIGN16: offset inputs */
  0, 34,                                         /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_siemens_800C */

/*--------------------------------------------------------------------------*/
/*--- AEG TIO 3344 ---------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_aeg_3344 [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 44,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 14,                                           /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x33, 0x44,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */
  0, 0, 0, 0, 0,                                           /* user_prm_data */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 4,                                            /* USIGN16: cfg_data_len */
  0x21,                                                         /* cfg_data */
  0x11,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  2,                                                       /* number_inputs */
  2,                                                      /* number_outputs */
  0x07, 0xFE,                                     /* USIGN16: offset inputs */
  0x07, 0xD0,                                    /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_aeg_3344 */

/*--------------------------------------------------------------------------*/
/*--- AEG TIO 1344 ---------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_aeg_1344 [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 44,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 14,                                           /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x13, 0x44,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */
  0, 0, 0, 0, 0,                                           /* user_prm_data */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 4,                                            /* USIGN16: cfg_data_len */
  0x00,                                                         /* cfg_data */
  0x11,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  2,                                                       /* number_inputs */
  0,                                                      /* number_outputs */
  0, 0,                                           /* USIGN16: offset inputs */
  0, 0,                                          /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_aeg_1344 */

/*--------------------------------------------------------------------------*/
/*--- SIEMENS ET200B 16DI --------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_siemens_0019 [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 44,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 14,                                           /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x00, 0x19,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */
  0, 0, 0, 0, 0,                                           /* user_prm_data */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 4,                                            /* USIGN16: cfg_data_len */
  0x00,                                                         /* cfg_data */
  0x11,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  2,                                                       /* number_inputs */
  0,                                                      /* number_outputs */
  0, 0,                                           /* USIGN16: offset inputs */
  0, 0,                                          /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_siemens_0019 */

/*--------------------------------------------------------------------------*/
/*--- SIEMENS ET200B 8DI/8DO -----------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_siemens_000B [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 44,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 14,                                           /* USIGN16: prm_data_len */
  DP_PRM_WD_ON,                                           /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x00, 0x0B,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */
  0, 0, 0, 0, 0,                                           /* user_prm_data */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 4,                                            /* USIGN16: cfg_data_len */
  0x20,                                                         /* cfg_data */
  0x10,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  1,                                                       /* number_inputs */
  1,                                                      /* number_outputs */
  0, 66,                                          /* USIGN16: offset inputs */
  0, 100,                                        /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_siemens_000B */

/*--------------------------------------------------------------------------*/
/*--- SIEMENS ET200B 16DI --------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_siemens_000E [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 44,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 14,                                           /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x00, 0x0E,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */
  0, 0, 0, 0, 0,                                           /* user_prm_data */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 4,                                            /* USIGN16: cfg_data_len */
  0x20,                                                         /* cfg_data */
  0x12,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  3,                                                       /* number_inputs */
  1,                                                      /* number_outputs */
  0, 102,                                         /* USIGN16: offset inputs */
  0, 68,                                         /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_siemens_0019 */

/*--------------------------------------------------------------------------*/
/*--- SIEMENS ET200M 16DO  (IM153-1) ---------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET     FAR dp_slave_siemens_801D [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 60,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                      /* reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 12,                                           /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  0,                                                            /* min_tsdr */
  0x80, 0x1D,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */
  0x40, 0x60, 0x00,                                        /* user_prm_data */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 22,                                           /* USIGN16: cfg_data_len */
  0x04,0x00,0x00,0xAD,0xC4,0x04,0x00,0x00,0x8B,0x41,0x04,0x00,0x00,                                                         /* cfg_data */
  0x8F,0xC0,0x83,0x01,0x00,0xAF,0xD0,

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  0,                                                       /* number_inputs */
  0,                                                      /* number_outputs */
  0, 0,                                           /* USIGN16: offset inputs */
  0, 0,                                           /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_siemens_0019 */

/*--------------------------------------------------------------------------*/
/*--- SOFTING PROFIboard/PROFI104 ------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET FAR dp_slave_softing_b205 [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 40,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                 /* .....reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 9,                                            /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  22,                                                           /* min_tsdr */
  0xB2, 0x05,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 5,                                           /*  USIGN16: cfg_data_len */
  0x10,                                      /* 1 byte input, byte consist. */
  0xD1,                                           /* 2 word input, consist. */
  0x21,                                     /* 2 byte output, byte consist. */

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  5,                                                       /* number_inputs */
  2,                                                      /* number_outputs */
  0, 2,                                           /* USIGN16: offset inputs */
  0, 8,                                          /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_softing_b205 */

/*--------------------------------------------------------------------------*/
/*--- DP_PARA_DESCR TABLE --------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static const OCTET FAR eh_1501 [] =
{
  /*--- SLAVE_PARA_SET -----------------------------------------------------*/

  0, 41,                                         /* USIGN16: slave_para_len */
  DP_SL_NEW_PRM | DP_SL_ACTIVE,                                  /* sl_flag */
  DP_SLAVE_TYPE_DP,                                           /* slave_type */
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,                 /* .....reserved [12] */

  /*--- PRM_DATA -----------------------------------------------------------*/

  0, 9,                                            /* USIGN16: prm_data_len */
  DP_PRM_LOCK_REQ | DP_PRM_WD_ON,                         /* station_status */
  100,                                                /* 10ms * wd_fact_1/2 */
  10,                                                            /* --> 10s */
  22,                                                           /* min_tsdr */
  0x15, 0x01,                                      /* USIGN16: ident_number */
  0,                                                         /* group_ident */

  /*--- CFG_DATA -----------------------------------------------------------*/

  0, 6,                                           /*  USIGN16: cfg_data_len */
  0x42,                                      
  0x84,                                          
  0x08,                                     
  0x05,                                     

  /*--- AAT_DATA -----------------------------------------------------------*/

  0, 8,                                            /* USIGN16: aat_data_len */
  5,                                                       /* number_inputs */
  2,                                                      /* number_outputs */
  0, 2,                                           /* USIGN16: offset inputs */
  0, 8,                                          /* USIGN16: offset outputs */

  /*--- SLAVE_USER_DATA ----------------------------------------------------*/

  0, 2,                                     /* USIGN16: slave_user_data_len */

}; /* dp_slave_softing_b205 */

/*--------------------------------------------------------------------------*/
/*--- DP_PARA_DESCR TABLE --------------------------------------------------*/
/*--------------------------------------------------------------------------*/


static const OCTET FAR* const slave_para_table [] =
{
  NULL,                                                               /*  0 */
  NULL,                                                               /*  1 */
  NULL,                                                               /*  2 */
  dp_slave_softing_b205,                                              /*  3 */
  dp_slave_softing_b205,                                              /*  4 */
  dp_slave_softing_b205,                                              /*  5 */
  NULL,                                                               /*  6 */
  NULL,                                                               /*  7 */
  NULL,                                                               /*  8 */
  NULL,                                                               /*  9 */
  NULL,                                                               /*  10 */
  NULL,                                                               /*  11 */
  eh_1501                                                               /*  12 */

};


/*--------------------------------------------------------------------------*/
/*--- STATION ADDRESSES OF ALL TEST SLAVES ---------------------------------*/
/*--------------------------------------------------------------------------*/

static const USIGN8   dp_slave_table [] =
{
  3, 4, 5, 12, 0xFF                       /* slave addresses; 0xFF == end */
};

/****************************************************************************/

#endif /* __DPDEMO__ */
