/*****************************************************************************/
/*                                                                           */
/*                                SOFTING AG                                 */
/*                         Richard-Reitzner-Allee 6                          */
/*                               D-85540 Haar                                */
/*                       Phone: ++49-89-4 56 56-0                            */
/*                         Fax: ++49-89-4 56 56-3 99                         */
/*                                                                           */
/*                     Copyright (C) SOFTING GmbH 1998-2003                  */
/*                            All Rights Reserved                            */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Filename    : DPSSCHED.H                                                 */
/*  Version     : 1.20                                                       */
/*  Date        : 30-04-2000                                                 */
/*  Author      : SOFTING AG                                                 */
/*                                                                           */
/*  Description : This file contains the structures, types and constants of  */
/*                file DPSSCHED.C                                            */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/

#ifndef __DPSSCHED__
#define __DPSSCHED__

/****************************************************************************/
/*** DEFINES ****************************************************************/
/****************************************************************************/

/*--- TIMER INTERRUPT DEFINITIONS ------------------------------------------*/

#define TIMER_INT   0x08
#define EOI_REG     0x20
#define EOI         0x20


/****************************************************************************/
/*** TYPES ******************************************************************/
/****************************************************************************/

/*--- SCHEDULER PRIORITY ---------------------------------------------------*/

typedef enum _T_DPSSCHED_TASK_PRIORITY
{
  DPSSCHED_NO_SCHEDULE = 0,
  DPSSCHED_PRIORITY_1,
  DPSSCHED_PRIORITY_2,
  DPSSCHED_PRIORITY_3,
  DPSSCHED_PRIORITY_4,
  DPSSCHED_PRIORITY_5,
  DPSSCHED_PRIORITY_6,
  DPSSCHED_PRIORITY_7,
  DPSSCHED_PRIORITY_8,
  DPSSCHED_PRIORITY_9,
  DPSSCHED_PRIORITY_10,
  DPSSCHED_MAX_NUMBER_PRIORITIES = 10

} T_DPSSCHED_TASK_PRIORITY;

/*--- SCHEDULER TASK IDENTIFICATION ----------------------------------------*/

typedef enum _T_DPSSCHED_TASK_ID
{
  DPSSCHED_INVALID_TASK_ID = -1,
  DPSSCHED_TIMER_TASK_ID = 0,
  DPSSCHED_KEYBOARD_TASK_ID,
  DPSSCHED_SERVICE_IF_TASK_ID,
  DPSSCHED_INPUT_IF_TASK_ID,
  DPSSCHED_OUTPUT_IF_TASK_ID,
  DPSSCHED_MAX_NUMBER_TASKS

} T_DPSSCHED_TASK_ID;

/*--- SCHEDULER EVENT IDENTIFICATION ---------------------------------------*/

typedef enum _T_DPSSCHED_EVENT_ID
{
  DPSSCHED_SCHEDULE_EVENT = 0,
  DPSSCHED_TIMER_EVENT,
  DPSSCHED_NEW_INPUT_EVENT,
  DPSSCHED_GET_STATUS_EVENT,
  DPSSCHED_MAX_NUMBER_EVENTS

} T_DPSSCHED_EVENT_ID;

/*--- DOUBLE LINKED LIST ELEMENT -------------------------------------------*/

typedef struct _T_LIST_ELEMENT
{
  struct _T_LIST_ELEMENT FAR*   next_ptr;
  struct _T_LIST_ELEMENT FAR*   prev_ptr;

  VOID FAR*                     data_ptr;

} T_LIST_ELEMENT;

/*--- DOUBLE LINKED LIST ---------------------------------------------------*/

typedef struct _T_LIST
{
  T_LIST_ELEMENT        head;
  T_LIST_ELEMENT        tail;

  T_LIST_ELEMENT FAR*   iterator_ptr;

  USIGN16               number_elements;

} T_LIST;

/****************************************************************************/
/*** FUNCTIONS **************************************************************/
/****************************************************************************/

FUNCTION extern PB_BOOL dpssched_init
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_exit
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_create_task
  (
  IN    T_DPSSCHED_TASK_ID          task_id,
  IN    T_DPSSCHED_TASK_PRIORITY    task_priority,
  IN    VOID                        (*task_function) (T_DPSSCHED_EVENT_ID)
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_start_schedule
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_stop_schedule
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_send_event
  (
  IN    T_DPSSCHED_TASK_ID    task_id,
  IN    T_DPSSCHED_EVENT_ID   event_id
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_start_timer
  (
  IN    T_DPSSCHED_TASK_ID    task_id,
  IN    USIGN32               duration,
  IN    PB_BOOL               cyclic
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID dpssched_stop_timer
  (
  IN    T_DPSSCHED_TASK_ID    task_id
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_element_init
  (
  IN    T_LIST_ELEMENT   FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - initializes the referenced list element                              */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_init
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - initializes the linking of the list base elements                    */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern T_LIST_ELEMENT FAR* list_get_first
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - returns the first element of the given list and resets the iterator  */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern T_LIST_ELEMENT FAR* list_get_last
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - returns the last element of the given list and resets the iterator   */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern T_LIST_ELEMENT FAR* list_get_next
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - returns the next available element of the double linked list         */
  /*   which is referenced by the iterator_ptr                              */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern T_LIST_ELEMENT FAR* list_get_prev
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - returns the prev available element of the double linked list         */
  /*   which is referenced by the iterator_ptr                              */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_add_head
  (
  IN    T_LIST           FAR *list_ptr,
  IN    T_LIST_ELEMENT   FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - inserts the referenced element at the head of a double linked list   */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_add_tail
  (
  IN    T_LIST           FAR *list_ptr,
  IN    T_LIST_ELEMENT   FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - inserts the referenced element at the end of a double linked list    */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_add_before
  (
  IN    T_LIST           FAR *list_ptr,
  IN    T_LIST_ELEMENT   FAR *position_ptr,
  IN    T_LIST_ELEMENT   FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - inserts the referenced element before the given position of a double */
  /*   linked list                                                          */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_add_after
  (
  IN    T_LIST           FAR *list_ptr,
  IN    T_LIST_ELEMENT   FAR *position_ptr,
  IN    T_LIST_ELEMENT   FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - inserts the referenced element after the given position of a double  */
  /*   linked list                                                          */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern T_LIST_ELEMENT FAR* list_remove_head
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - removes the element at the head of a double linked list              */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern T_LIST_ELEMENT FAR* list_remove_tail
  (
  IN    T_LIST    FAR *list_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - removes the element at the end of a double linked list               */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern VOID list_remove_at
  (
  IN    T_LIST            FAR *list_ptr,
  IN    T_LIST_ELEMENT    FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - removes the referenced element from the given list                   */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern PB_BOOL list_find
  (
  IN    T_LIST            FAR *list_ptr,
  IN    T_LIST_ELEMENT    FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - checks whether the referenced list element is contained in the list  */
  /*------------------------------------------------------------------------*/
;

FUNCTION extern PB_BOOL list_find_remove
  (
  IN    T_LIST            FAR *list_ptr,
  IN    T_LIST_ELEMENT    FAR *element_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* - checks whether the referenced list element is contained in the list  */
  /* - removes the element when found                                       */
  /*------------------------------------------------------------------------*/
;

/*** END OF DPSSched.c ******************************************************/

#endif /* __DPSSCHED__ */
